from _typeshed import Incomplete, SupportsItems, SupportsKeysAndGetItem, Unused
from collections.abc import Generator, Hashable, Iterable, Iterator
from typing import Literal, TypeVar, overload

from networkx.algorithms.planarity import PlanarEmbedding
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _NBunch, _Node
from networkx.classes.multigraph import MultiGraph

__all__ = [
    "nodes",
    "edges",
    "degree",
    "degree_histogram",
    "neighbors",
    "number_of_nodes",
    "number_of_edges",
    "density",
    "is_directed",
    "freeze",
    "is_frozen",
    "subgraph",
    "induced_subgraph",
    "edge_subgraph",
    "restricted_view",
    "to_directed",
    "to_undirected",
    "add_star",
    "add_path",
    "add_cycle",
    "create_empty_copy",
    "set_node_attributes",
    "get_node_attributes",
    "set_edge_attributes",
    "get_edge_attributes",
    "all_neighbors",
    "non_neighbors",
    "non_edges",
    "common_neighbors",
    "is_weighted",
    "is_negatively_weighted",
    "is_empty",
    "selfloop_edges",
    "nodes_with_selfloops",
    "number_of_selfloops",
    "path_weight",
    "is_path",
]
_U = TypeVar("_U")

def nodes(G): ...
def edges(G, nbunch: Incomplete | None = None): ...
def degree(G, nbunch: Incomplete | None = None, weight: Incomplete | None = None): ...
def neighbors(G, n): ...
def number_of_nodes(G): ...
def number_of_edges(G): ...
def density(G): ...
def degree_histogram(G): ...
@overload
def is_directed(G: PlanarEmbedding[Hashable]) -> Literal[False]: ...  # type: ignore[misc] # Incompatible return types
@overload
def is_directed(G: DiGraph[Hashable]) -> Literal[True]: ...  # type: ignore[misc] # Incompatible return types
@overload
def is_directed(G: Graph[Hashable]) -> Literal[False]: ...
def freeze(G): ...
def is_frozen(G): ...
def add_star(G_to_add_to, nodes_for_star, **attr) -> None: ...
def add_path(G_to_add_to, nodes_for_path, **attr) -> None: ...
def add_cycle(G_to_add_to, nodes_for_cycle, **attr) -> None: ...
def subgraph(G, nbunch): ...
def induced_subgraph(G: Graph[_Node], nbunch: _NBunch[_Node]) -> Graph[_Node]: ...
def edge_subgraph(G, edges): ...
def restricted_view(G, nodes, edges): ...
def to_directed(graph): ...
def to_undirected(graph): ...
def create_empty_copy(G, with_data: bool = True): ...

# incomplete: Can "Any scalar value" be enforced?
@overload
def set_node_attributes(G: Graph[Hashable], values: SupportsItems[_Node, Unused], name: str) -> None: ...
@overload
def set_node_attributes(
    G: Graph[_Node],
    values: SupportsItems[_Node, SupportsKeysAndGetItem[Incomplete, Incomplete] | Iterable[tuple[Incomplete, Incomplete]]],
    name: None = None,
) -> None: ...
def get_node_attributes(G: Graph[_Node], name: str, default=None) -> dict[_Node, Incomplete]: ...
@overload
def set_edge_attributes(G: Graph[_Node], values: SupportsItems[tuple[_Node, _Node], Incomplete], name: str) -> None: ...
@overload
def set_edge_attributes(G: MultiGraph[_Node], values: dict[tuple[_Node, _Node, Incomplete], Incomplete], name: str) -> None: ...
@overload
def set_edge_attributes(G: Graph[Hashable], values, name: None = None) -> None: ...
def get_edge_attributes(G: Graph[_Node], name: str, default=None) -> dict[tuple[_Node, _Node], Incomplete]: ...
def all_neighbors(graph: Graph[_Node], node: _Node) -> Iterator[_Node]: ...
def non_neighbors(graph: Graph[_Node], node: _Node) -> Generator[_Node, None, None]: ...
def non_edges(graph: Graph[_Node]) -> Generator[tuple[_Node, _Node], None, None]: ...
def common_neighbors(G: Graph[_Node], u: _Node, v: _Node) -> Generator[_Node, None, None]: ...
def is_weighted(G: Graph[_Node], edge: tuple[_Node, _Node] | None = None, weight: str = "weight") -> bool: ...
def is_negatively_weighted(G: Graph[_Node], edge: tuple[_Node, _Node] | None = None, weight: str = "weight"): ...
def is_empty(G: Graph[Hashable]) -> bool: ...
def nodes_with_selfloops(G: Graph[_Node]) -> Generator[_Node, None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False] = False, keys: Literal[False] = False, default=None
) -> Generator[tuple[_Node, _Node], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[True], keys: Literal[False] = False, default=None
) -> Generator[tuple[_Node, _Node, dict[str, Incomplete]], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: str, keys: Literal[False] = False, default: _U | None = None
) -> Generator[tuple[_Node, _Node, _U], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False], keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False] = False, *, keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[True], keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int, dict[str, Incomplete]], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: str, keys: Literal[True], default: _U | None = None
) -> Generator[tuple[_Node, _Node, int, _U], None, None]: ...
def number_of_selfloops(G: Graph[Hashable]) -> int: ...
def is_path(G, path) -> bool: ...
def path_weight(G, path, weight) -> int: ...



from _typeshed import Incomplete
from typing import Any

def load_endpoint_json(path): ...
def merge_endpoints(defaults, additions): ...
def load_regions(): ...
def get_regions(service_name, region_cls: Incomplete | None = None, connection_cls: Incomplete | None = None): ...

class RegionInfo:
    connection: Any
    name: Any
    endpoint: Any
    connection_cls: Any
    def __init__(
        self,
        connection: Incomplete | None = None,
        name: Incomplete | None = None,
        endpoint: Incomplete | None = None,
        connection_cls: Incomplete | None = None,
    ) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def connect(self, **kw_params): ...



from networkx.utils.backends import _dispatch

@_dispatch
def kl_connected_subgraph(G, k, l, low_memory: bool = False, same_as_graph: bool = False): ...
@_dispatch
def is_kl_connected(G, k, l, low_memory: bool = False): ...



import pygit2.enums

class SearchPathList:
    def __getitem__(self, key: int) -> str: ...
    def __setitem__(self, key: int, value: bytes | str) -> None: ...

class Settings:
    def __init__(self) -> None: ...
    @property
    def search_path(self) -> SearchPathList: ...
    @property
    def mwindow_size(self) -> int: ...
    @mwindow_size.setter
    def mwindow_size(self, value: int) -> None: ...
    @property
    def mwindow_mapped_limit(self) -> int: ...
    @mwindow_mapped_limit.setter
    def mwindow_mapped_limit(self, value: int) -> None: ...
    @property
    def cached_memory(self) -> tuple[int, int]: ...
    def enable_caching(self, value: bool = True) -> None: ...
    def disable_pack_keep_file_checks(self, value: bool = True) -> None: ...
    def cache_max_size(self, value: int) -> None: ...
    def cache_object_limit(self, object_type: pygit2.enums.ObjectType, value: int) -> None: ...
    @property
    def ssl_cert_file(self) -> bytes | str: ...
    @ssl_cert_file.setter
    def ssl_cert_file(self, value: bytes | str) -> None: ...
    @ssl_cert_file.deleter
    def ssl_cert_file(self) -> None: ...
    @property
    def ssl_cert_dir(self) -> bytes | str: ...
    @ssl_cert_dir.setter
    def ssl_cert_dir(self, value: bytes | str) -> None: ...
    @ssl_cert_dir.deleter
    def ssl_cert_dir(self) -> None: ...
    def set_ssl_cert_locations(self, cert_file: bytes | str, cert_dir: bytes | str) -> None: ...



from _typeshed import Incomplete
from typing import Protocol

from docutils.utils import Reporter

class _LanguageModule(Protocol):
    labels: dict[str, str]
    author_separators: list[str]
    bibliographic_fields: list[str]

class LanguageImporter:
    def __call__(self, language_code: str, reporter: Reporter | None = None) -> _LanguageModule: ...
    def __getattr__(self, name: str, /) -> Incomplete: ...

get_language: LanguageImporter



from _typeshed import Incomplete, Unused
from collections.abc import AsyncIterator, Awaitable, Callable, Generator, Iterable, Mapping, MutableMapping, Sequence
from datetime import datetime, timedelta
from types import TracebackType
from typing import Any, ClassVar, Literal, NoReturn, Protocol, TypedDict, overload
from typing_extensions import Self, TypeAlias

from redis import RedisError
from redis.asyncio.connection import ConnectCallbackT, Connection, ConnectionPool
from redis.asyncio.lock import Lock
from redis.asyncio.retry import Retry
from redis.client import AbstractRedis, _CommandOptions, _Key, _StrType, _Value
from redis.commands import AsyncCoreCommands, AsyncSentinelCommands, RedisModuleCommands
from redis.credentials import CredentialProvider
from redis.typing import ChannelT, EncodableT, KeyT, PatternT, StreamIdT

PubSubHandler: TypeAlias = Callable[[dict[str, str]], Awaitable[None]]

class ResponseCallbackProtocol(Protocol):
    def __call__(self, response: Any, **kwargs): ...

class AsyncResponseCallbackProtocol(Protocol):
    async def __call__(self, response: Any, **kwargs): ...

ResponseCallbackT: TypeAlias = ResponseCallbackProtocol | AsyncResponseCallbackProtocol

class Redis(AbstractRedis, RedisModuleCommands, AsyncCoreCommands[_StrType], AsyncSentinelCommands):
    response_callbacks: MutableMapping[str | bytes, ResponseCallbackT]
    auto_close_connection_pool: bool
    connection_pool: Any
    single_connection_client: Any
    connection: Any
    @overload
    @classmethod
    def from_url(
        cls,
        url: str,
        *,
        host: str = "localhost",
        port: int = 6379,
        db: str | int = 0,
        password: str | None = None,
        socket_timeout: float | None = None,
        socket_connect_timeout: float | None = None,
        socket_keepalive: bool | None = None,
        socket_keepalive_options: Mapping[int, int | bytes] | None = None,
        connection_pool: ConnectionPool | None = None,
        unix_socket_path: str | None = None,
        encoding: str = "utf-8",
        encoding_errors: str = "strict",
        decode_responses: Literal[True],
        retry_on_timeout: bool = False,
        retry_on_error: list[type[RedisError]] | None = None,
        ssl: bool = False,
        ssl_keyfile: str | None = None,
        ssl_certfile: str | None = None,
        ssl_cert_reqs: str = "required",
        ssl_ca_certs: str | None = None,
        ssl_ca_data: str | None = None,
        ssl_check_hostname: bool = False,
        max_connections: int | None = None,
        single_connection_client: bool = False,
        health_check_interval: int = 0,
        client_name: str | None = None,
        username: str | None = None,
        retry: Retry | None = None,
        auto_close_connection_pool: bool = True,
        redis_connect_func: ConnectCallbackT | None = None,
        credential_provider: CredentialProvider | None = None,
    ) -> Redis[str]: ...
    @overload
    @classmethod
    def from_url(
        cls,
        url: str,
        *,
        host: str = "localhost",
        port: int = 6379,
        db: str | int = 0,
        password: str | None = None,
        socket_timeout: float | None = None,
        socket_connect_timeout: float | None = None,
        socket_keepalive: bool | None = None,
        socket_keepalive_options: Mapping[int, int | bytes] | None = None,
        connection_pool: ConnectionPool | None = None,
        unix_socket_path: str | None = None,
        encoding: str = "utf-8",
        encoding_errors: str = "strict",
        decode_responses: Literal[False] = False,
        retry_on_timeout: bool = False,
        retry_on_error: list[type[RedisError]] | None = None,
        ssl: bool = False,
        ssl_keyfile: str | None = None,
        ssl_certfile: str | None = None,
        ssl_cert_reqs: str = "required",
        ssl_ca_certs: str | None = None,
        ssl_ca_data: str | None = None,
        ssl_check_hostname: bool = False,
        max_connections: int | None = None,
        single_connection_client: bool = False,
        health_check_interval: int = 0,
        client_name: str | None = None,
        username: str | None = None,
        retry: Retry | None = None,
        auto_close_connection_pool: bool = True,
        redis_connect_func: ConnectCallbackT | None = None,
        credential_provider: CredentialProvider | None = None,
    ) -> Redis[bytes]: ...
    @overload
    def __init__(
        self: Redis[str],
        *,
        host: str = "localhost",
        port: int = 6379,
        db: str | int = 0,
        password: str | None = None,
        socket_timeout: float | None = None,
        socket_connect_timeout: float | None = None,
        socket_keepalive: bool | None = None,
        socket_keepalive_options: Mapping[int, int | bytes] | None = None,
        connection_pool: ConnectionPool | None = None,
        unix_socket_path: str | None = None,
        encoding: str = "utf-8",
        encoding_errors: str = "strict",
        decode_responses: Literal[True],
        retry_on_timeout: bool = False,
        retry_on_error: list[type[RedisError]] | None = None,
        ssl: bool = False,
        ssl_keyfile: str | None = None,
        ssl_certfile: str | None = None,
        ssl_cert_reqs: str = "required",
        ssl_ca_certs: str | None = None,
        ssl_ca_data: str | None = None,
        ssl_check_hostname: bool = False,
        max_connections: int | None = None,
        single_connection_client: bool = False,
        health_check_interval: int = 0,
        client_name: str | None = None,
        username: str | None = None,
        retry: Retry | None = None,
        auto_close_connection_pool: bool = True,
        redis_connect_func: ConnectCallbackT | None = None,
        credential_provider: CredentialProvider | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: Redis[bytes],
        *,
        host: str = "localhost",
        port: int = 6379,
        db: str | int = 0,
        password: str | None = None,
        socket_timeout: float | None = None,
        socket_connect_timeout: float | None = None,
        socket_keepalive: bool | None = None,
        socket_keepalive_options: Mapping[int, int | bytes] | None = None,
        connection_pool: ConnectionPool | None = None,
        unix_socket_path: str | None = None,
        encoding: str = "utf-8",
        encoding_errors: str = "strict",
        decode_responses: Literal[False] = False,
        retry_on_timeout: bool = False,
        retry_on_error: list[type[RedisError]] | None = None,
        ssl: bool = False,
        ssl_keyfile: str | None = None,
        ssl_certfile: str | None = None,
        ssl_cert_reqs: str = "required",
        ssl_ca_certs: str | None = None,
        ssl_ca_data: str | None = None,
        ssl_check_hostname: bool = False,
        max_connections: int | None = None,
        single_connection_client: bool = False,
        health_check_interval: int = 0,
        client_name: str | None = None,
        username: str | None = None,
        retry: Retry | None = None,
        auto_close_connection_pool: bool = True,
        redis_connect_func: ConnectCallbackT | None = None,
        credential_provider: CredentialProvider | None = None,
    ) -> None: ...
    def __await__(self) -> Generator[Any, None, Self]: ...
    async def initialize(self) -> Self: ...
    def set_response_callback(self, command: str, callback: ResponseCallbackT): ...
    def load_external_module(self, funcname, func) -> None: ...
    def pipeline(self, transaction: bool = True, shard_hint: str | None = None) -> Pipeline[_StrType]: ...
    async def transaction(
        self,
        func: Callable[[Pipeline[_StrType]], Any | Awaitable[Any]],
        *watches: KeyT,
        shard_hint: str | None = None,
        value_from_callable: bool = False,
        watch_delay: float | None = None,
    ): ...
    def lock(
        self,
        name: KeyT,
        timeout: float | None = None,
        sleep: float = 0.1,
        blocking: bool = True,
        blocking_timeout: float | None = None,
        lock_class: type[Lock] | None = None,
        thread_local: bool = True,
    ) -> Lock: ...
    def pubsub(self, **kwargs) -> PubSub: ...
    def monitor(self) -> Monitor: ...
    def client(self) -> Redis[_StrType]: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self, _warnings: Any = ...) -> None: ...
    async def close(self, close_connection_pool: bool | None = None) -> None: ...
    async def execute_command(self, *args, **options): ...
    async def parse_response(self, connection: Connection, command_name: str | bytes, **options): ...

StrictRedis = Redis

class MonitorCommandInfo(TypedDict):
    time: float
    db: int
    client_address: str
    client_port: str
    client_type: str
    command: str

class Monitor:
    monitor_re: Any
    command_re: Any
    connection_pool: Any
    connection: Any
    def __init__(self, connection_pool: ConnectionPool) -> None: ...
    async def connect(self) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(self, *args: Unused) -> None: ...
    async def next_command(self) -> MonitorCommandInfo: ...
    def listen(self) -> AsyncIterator[MonitorCommandInfo]: ...

class PubSub:
    PUBLISH_MESSAGE_TYPES: ClassVar[tuple[str, ...]]
    UNSUBSCRIBE_MESSAGE_TYPES: ClassVar[tuple[str, ...]]
    HEALTH_CHECK_MESSAGE: ClassVar[str]
    connection_pool: Any
    shard_hint: str | None
    ignore_subscribe_messages: bool
    connection: Any
    encoder: Any
    health_check_response: Iterable[str | bytes]
    channels: Any
    pending_unsubscribe_channels: Any
    patterns: Any
    pending_unsubscribe_patterns: Any
    def __init__(
        self,
        connection_pool: ConnectionPool,
        shard_hint: str | None = None,
        ignore_subscribe_messages: bool = False,
        encoder: Incomplete | None = None,
    ) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...
    async def reset(self) -> None: ...
    def close(self) -> Awaitable[NoReturn]: ...
    async def on_connect(self, connection: Connection): ...
    @property
    def subscribed(self) -> bool: ...
    async def execute_command(self, *args: EncodableT): ...
    async def parse_response(self, block: bool = True, timeout: float = 0): ...
    async def check_health(self) -> None: ...
    async def psubscribe(self, *args: ChannelT, **kwargs: PubSubHandler): ...
    def punsubscribe(self, *args: ChannelT) -> Awaitable[Any]: ...
    async def subscribe(self, *args: ChannelT, **kwargs: Callable[..., Any]): ...
    def unsubscribe(self, *args) -> Awaitable[Any]: ...
    def listen(self) -> AsyncIterator[Any]: ...
    async def get_message(self, ignore_subscribe_messages: bool = False, timeout: float = 0.0): ...
    def ping(self, message: Incomplete | None = None) -> Awaitable[Any]: ...
    async def handle_message(self, response, ignore_subscribe_messages: bool = False): ...
    async def run(self, *, exception_handler: PSWorkerThreadExcHandlerT | None = None, poll_timeout: float = 1.0) -> None: ...

class PubsubWorkerExceptionHandler(Protocol):
    def __call__(self, e: BaseException, pubsub: PubSub): ...

class AsyncPubsubWorkerExceptionHandler(Protocol):
    async def __call__(self, e: BaseException, pubsub: PubSub): ...

PSWorkerThreadExcHandlerT: TypeAlias = PubsubWorkerExceptionHandler | AsyncPubsubWorkerExceptionHandler
CommandT: TypeAlias = tuple[tuple[str | bytes, ...], Mapping[str, Any]]
CommandStackT: TypeAlias = list[CommandT]

class Pipeline(Redis[_StrType]):
    UNWATCH_COMMANDS: ClassVar[set[str]]
    connection_pool: Any
    connection: Any
    response_callbacks: Any
    is_transaction: bool
    shard_hint: str | None
    watching: bool
    command_stack: Any
    scripts: Any
    explicit_transaction: bool
    def __init__(
        self,
        connection_pool: ConnectionPool,
        response_callbacks: MutableMapping[str | bytes, ResponseCallbackT],
        transaction: bool,
        shard_hint: str | None,
    ) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __await__(self) -> Generator[Any, None, Self]: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    async def reset(self) -> None: ...
    def multi(self) -> None: ...
    def execute_command(self, *args, **kwargs) -> Pipeline[_StrType] | Awaitable[Pipeline[_StrType]]: ...
    async def immediate_execute_command(self, *args, **options): ...
    def pipeline_execute_command(self, *args, **options): ...
    def raise_first_error(self, commands: CommandStackT, response: Iterable[Any]): ...
    def annotate_exception(self, exception: Exception, number: int, command: Iterable[object]) -> None: ...
    async def parse_response(self, connection: Connection, command_name: str | bytes, **options): ...
    async def load_scripts(self) -> None: ...
    async def execute(self, raise_on_error: bool = True): ...
    async def discard(self) -> None: ...
    async def watch(self, *names: KeyT) -> bool: ...
    async def unwatch(self) -> bool: ...
    # region acl commands
    def acl_cat(self, category: str | None = None, **kwargs: _CommandOptions) -> Any: ...
    def acl_deluser(self, *username: str, **kwargs: _CommandOptions) -> Any: ...
    def acl_genpass(self, bits: int | None = None, **kwargs: _CommandOptions) -> Any: ...
    def acl_getuser(self, username: str, **kwargs: _CommandOptions) -> Any: ...
    def acl_help(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_list(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_log(self, count: int | None = None, **kwargs: _CommandOptions) -> Any: ...
    def acl_log_reset(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_load(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_save(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_setuser(  # type: ignore[override]
        self,
        username: str,
        enabled: bool = False,
        nopass: bool = False,
        passwords: Sequence[str] | None = None,
        hashed_passwords: Sequence[str] | None = None,
        categories: Sequence[str] | None = None,
        commands: Sequence[str] | None = None,
        keys: Sequence[str] | None = None,
        channels: Iterable[ChannelT] | None = None,
        selectors: Iterable[tuple[str, KeyT]] | None = None,
        reset: bool = False,
        reset_keys: bool = False,
        reset_channels: bool = False,
        reset_passwords: bool = False,
        **kwargs: _CommandOptions,
    ) -> Pipeline[_StrType]: ...
    def acl_users(self, **kwargs: _CommandOptions) -> Any: ...
    def acl_whoami(self, **kwargs: _CommandOptions) -> Any: ...
    # endregion
    # region cluster commands
    def cluster(self, cluster_arg: str, *args, **kwargs: _CommandOptions) -> Any: ...
    def readwrite(self, **kwargs: _CommandOptions) -> Any: ...
    def readonly(self, **kwargs: _CommandOptions) -> Any: ...
    # endregion
    # region BasicKey commands
    def append(self, key, value) -> Any: ...
    def bitcount(self, key: _Key, start: int | None = None, end: int | None = None, mode: str | None = None) -> Any: ...
    def bitfield(self, key, default_overflow: Incomplete | None = None) -> Any: ...
    def bitop(self, operation, dest, *keys) -> Any: ...
    def bitpos(self, key: _Key, bit: int, start: int | None = None, end: int | None = None, mode: str | None = None) -> Any: ...
    def copy(self, source, destination, destination_db: Incomplete | None = None, replace: bool = False) -> Any: ...
    def decr(self, name, amount: int = 1) -> Any: ...
    def decrby(self, name, amount: int = 1) -> Any: ...
    def delete(self, *names: _Key) -> Any: ...
    def dump(self, name: _Key) -> Any: ...
    def exists(self, *names: _Key) -> Any: ...
    def expire(
        self, name: _Key, time: int | timedelta, nx: bool = False, xx: bool = False, gt: bool = False, lt: bool = False
    ) -> Any: ...
    def expireat(self, name, when, nx: bool = False, xx: bool = False, gt: bool = False, lt: bool = False) -> Any: ...
    def get(self, name: _Key) -> Any: ...
    def getdel(self, name: _Key) -> Any: ...
    def getex(
        self,
        name,
        ex: Incomplete | None = None,
        px: Incomplete | None = None,
        exat: Incomplete | None = None,
        pxat: Incomplete | None = None,
        persist: bool = False,
    ) -> Any: ...
    def getbit(self, name: _Key, offset: int) -> Any: ...
    def getrange(self, key, start, end) -> Any: ...
    def getset(self, name, value) -> Any: ...
    def incr(self, name: _Key, amount: int = 1) -> Any: ...
    def incrby(self, name: _Key, amount: int = 1) -> Any: ...
    def incrbyfloat(self, name: _Key, amount: float = 1.0) -> Any: ...
    def keys(self, pattern: _Key = "*", **kwargs: _CommandOptions) -> Any: ...
    def lmove(
        self,
        first_list: _Key,
        second_list: _Key,
        src: Literal["LEFT", "RIGHT"] = "LEFT",
        dest: Literal["LEFT", "RIGHT"] = "RIGHT",
    ) -> Any: ...
    def blmove(
        self,
        first_list: _Key,
        second_list: _Key,
        timeout: float,
        src: Literal["LEFT", "RIGHT"] = "LEFT",
        dest: Literal["LEFT", "RIGHT"] = "RIGHT",
    ) -> Any: ...
    def mget(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def mset(self, mapping: Mapping[_Key, _Value]) -> Any: ...
    def msetnx(self, mapping: Mapping[_Key, _Value]) -> Any: ...
    def move(self, name: _Key, db: int) -> Any: ...
    def persist(self, name: _Key) -> Any: ...
    def pexpire(
        self, name: _Key, time: int | timedelta, nx: bool = False, xx: bool = False, gt: bool = False, lt: bool = False
    ) -> Any: ...
    def pexpireat(
        self, name: _Key, when: int | datetime, nx: bool = False, xx: bool = False, gt: bool = False, lt: bool = False
    ) -> Any: ...
    def psetex(self, name, time_ms, value) -> Any: ...
    def pttl(self, name: _Key) -> Any: ...
    def hrandfield(self, key, count: Incomplete | None = None, withvalues: bool = False) -> Any: ...
    def randomkey(self, **kwargs: _CommandOptions) -> Any: ...
    def rename(self, src, dst) -> Any: ...
    def renamenx(self, src, dst) -> Any: ...
    def restore(
        self,
        name,
        ttl,
        value,
        replace: bool = False,
        absttl: bool = False,
        idletime: Incomplete | None = None,
        frequency: Incomplete | None = None,
    ) -> Any: ...
    def set(  # type: ignore[override]
        self,
        name: _Key,
        value: _Value,
        ex: None | int | timedelta = None,
        px: None | int | timedelta = None,
        nx: bool = False,
        xx: bool = False,
        keepttl: bool = False,
        get: bool = False,
        exat: Incomplete | None = None,
        pxat: Incomplete | None = None,
    ) -> Any: ...
    def setbit(self, name: _Key, offset: int, value: int) -> Any: ...
    def setex(self, name: _Key, time: int | timedelta, value: _Value) -> Any: ...
    def setnx(self, name: _Key, value: _Value) -> Any: ...
    def setrange(self, name, offset, value) -> Any: ...
    def stralgo(
        self,
        algo,
        value1,
        value2,
        specific_argument: str = "strings",
        len: bool = False,
        idx: bool = False,
        minmatchlen: Incomplete | None = None,
        withmatchlen: bool = False,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def strlen(self, name) -> Any: ...
    def substr(self, name, start, end: int = -1) -> Any: ...
    def touch(self, *args) -> Any: ...
    def ttl(self, name: _Key) -> Any: ...
    def type(self, name) -> Any: ...
    def unlink(self, *names: _Key) -> Any: ...
    # endregion
    # region hyperlog commands
    def pfadd(self, name: _Key, *values: _Value) -> Any: ...
    def pfcount(self, name: _Key) -> Any: ...
    def pfmerge(self, dest: _Key, *sources: _Key) -> Any: ...
    # endregion
    # region hash commands
    def hdel(self, name: _Key, *keys: _Key) -> Any: ...
    def hexists(self, name: _Key, key: _Key) -> Any: ...
    def hget(self, name: _Key, key: _Key) -> Any: ...
    def hgetall(self, name: _Key) -> Any: ...
    def hincrby(self, name: _Key, key: _Key, amount: int = 1) -> Any: ...
    def hincrbyfloat(self, name: _Key, key: _Key, amount: float = 1.0) -> Any: ...
    def hkeys(self, name: _Key) -> Any: ...
    def hlen(self, name: _Key) -> Any: ...
    @overload
    def hset(
        self, name: _Key, key: _Key, value: _Value, mapping: Mapping[_Key, _Value] | None = None, items: Incomplete | None = None
    ) -> Any: ...
    @overload
    def hset(
        self, name: _Key, key: None, value: None, mapping: Mapping[_Key, _Value], items: Incomplete | None = None
    ) -> Any: ...
    @overload
    def hset(self, name: _Key, *, mapping: Mapping[_Key, _Value], items: Incomplete | None = None) -> Any: ...
    def hsetnx(self, name: _Key, key: _Key, value: _Value) -> Any: ...
    def hmset(self, name: _Key, mapping: Mapping[_Key, _Value]) -> Any: ...
    def hmget(self, name: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def hvals(self, name: _Key) -> Any: ...
    def hstrlen(self, name, key) -> Any: ...
    # endregion
    # region geo commands
    def geoadd(self, name, values, nx: bool = False, xx: bool = False, ch: bool = False) -> Any: ...
    def geodist(self, name, place1, place2, unit: Incomplete | None = None) -> Any: ...
    def geohash(self, name, *values) -> Any: ...
    def geopos(self, name, *values) -> Any: ...
    def georadius(
        self,
        name,
        longitude,
        latitude,
        radius,
        unit: Incomplete | None = None,
        withdist: bool = False,
        withcoord: bool = False,
        withhash: bool = False,
        count: Incomplete | None = None,
        sort: Incomplete | None = None,
        store: Incomplete | None = None,
        store_dist: Incomplete | None = None,
        any: bool = False,
    ) -> Any: ...
    def georadiusbymember(
        self,
        name,
        member,
        radius,
        unit: Incomplete | None = None,
        withdist: bool = False,
        withcoord: bool = False,
        withhash: bool = False,
        count: Incomplete | None = None,
        sort: Incomplete | None = None,
        store: Incomplete | None = None,
        store_dist: Incomplete | None = None,
        any: bool = False,
    ) -> Any: ...
    def geosearch(
        self,
        name,
        member: Incomplete | None = None,
        longitude: Incomplete | None = None,
        latitude: Incomplete | None = None,
        unit: str = "m",
        radius: Incomplete | None = None,
        width: Incomplete | None = None,
        height: Incomplete | None = None,
        sort: Incomplete | None = None,
        count: Incomplete | None = None,
        any: bool = False,
        withcoord: bool = False,
        withdist: bool = False,
        withhash: bool = False,
    ) -> Any: ...
    def geosearchstore(
        self,
        dest,
        name,
        member: Incomplete | None = None,
        longitude: Incomplete | None = None,
        latitude: Incomplete | None = None,
        unit: str = "m",
        radius: Incomplete | None = None,
        width: Incomplete | None = None,
        height: Incomplete | None = None,
        sort: Incomplete | None = None,
        count: Incomplete | None = None,
        any: bool = False,
        storedist: bool = False,
    ) -> Any: ...
    # endregion
    # region list commands
    @overload
    def blpop(self, keys: _Value | Iterable[_Value], timeout: Literal[0] | None = 0) -> Any: ...
    @overload
    def blpop(self, keys: _Value | Iterable[_Value], timeout: float) -> Any: ...
    @overload
    def brpop(self, keys: _Value | Iterable[_Value], timeout: Literal[0] | None = 0) -> Any: ...
    @overload
    def brpop(self, keys: _Value | Iterable[_Value], timeout: float) -> Any: ...
    def brpoplpush(self, src, dst, timeout: int | None = 0) -> Any: ...
    def lindex(self, name: _Key, index: int) -> Any: ...
    def linsert(
        self, name: _Key, where: Literal["BEFORE", "AFTER", "before", "after"], refvalue: _Value, value: _Value
    ) -> Any: ...
    def llen(self, name: _Key) -> Any: ...
    def lpop(self, name, count: int | None = None) -> Any: ...
    def lpush(self, name: _Value, *values: _Value) -> Any: ...
    def lpushx(self, name, value) -> Any: ...
    def lrange(self, name: _Key, start: int, end: int) -> Any: ...
    def lrem(self, name: _Key, count: int, value: _Value) -> Any: ...
    def lset(self, name: _Key, index: int, value: _Value) -> Any: ...
    def ltrim(self, name: _Key, start: int, end: int) -> Any: ...
    def rpop(self, name, count: int | None = None) -> Any: ...
    def rpoplpush(self, src, dst) -> Any: ...
    def rpush(self, name: _Value, *values: _Value) -> Any: ...
    def rpushx(self, name, value) -> Any: ...
    def lpos(
        self, name, value, rank: Incomplete | None = None, count: Incomplete | None = None, maxlen: Incomplete | None = None
    ) -> Any: ...
    @overload  # type: ignore[override]
    def sort(
        self,
        name: _Key,
        start: int | None = None,
        num: int | None = None,
        by: _Key | None = None,
        get: _Key | Sequence[_Key] | None = None,
        desc: bool = False,
        alpha: bool = False,
        store: None = None,
        groups: bool = False,
    ) -> list[_StrType]: ...
    @overload
    def sort(
        self,
        name: _Key,
        start: int | None = None,
        num: int | None = None,
        by: _Key | None = None,
        get: _Key | Sequence[_Key] | None = None,
        desc: bool = False,
        alpha: bool = False,
        *,
        store: _Key,
        groups: bool = False,
    ) -> Any: ...
    @overload
    def sort(
        self,
        name: _Key,
        start: int | None,
        num: int | None,
        by: _Key | None,
        get: _Key | Sequence[_Key] | None,
        desc: bool,
        alpha: bool,
        store: _Key,
        groups: bool = False,
    ) -> Any: ...
    # endregion
    # region scan commands
    def scan(
        self,
        cursor: int = 0,
        match: _Key | None = None,
        count: int | None = None,
        _type: str | None = None,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def sscan(self, name: _Key, cursor: int = 0, match: _Key | None = None, count: int | None = None) -> Any: ...
    def hscan(self, name: _Key, cursor: int = 0, match: _Key | None = None, count: int | None = None) -> Any: ...
    @overload
    def zscan(self, name: _Key, cursor: int = 0, match: _Key | None = None, count: int | None = None) -> Any: ...
    @overload
    def zscan(
        self,
        name: _Key,
        cursor: int = 0,
        match: _Key | None = None,
        count: int | None = None,
        *,
        score_cast_func: Callable[[_StrType], Any],
    ) -> Any: ...
    @overload
    def zscan(
        self, name: _Key, cursor: int, match: _Key | None, count: int | None, score_cast_func: Callable[[_StrType], Any]
    ) -> Any: ...
    # endregion
    # region set commands
    def sadd(self, name: _Key, *values: _Value) -> Any: ...
    def scard(self, name: _Key) -> Any: ...
    def sdiff(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def sdiffstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def sinter(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def sinterstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def sismember(self, name: _Key, value: _Value) -> Any: ...
    def smembers(self, name: _Key) -> Any: ...
    def smismember(self, name, values, *args) -> Any: ...
    def smove(self, src: _Key, dst: _Key, value: _Value) -> Any: ...
    @overload
    def spop(self, name: _Key, count: None = None) -> Any: ...
    @overload
    def spop(self, name: _Key, count: int) -> Any: ...
    @overload
    def srandmember(self, name: _Key, number: None = None) -> Any: ...
    @overload
    def srandmember(self, name: _Key, number: int) -> Any: ...
    def srem(self, name: _Key, *values: _Value) -> Any: ...
    def sunion(self, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    def sunionstore(self, dest: _Key, keys: _Key | Iterable[_Key], *args: _Key) -> Any: ...
    # endregion
    # region stream commands
    def xack(self, name, groupname, *ids) -> Any: ...
    def xadd(
        self,
        name,
        fields,
        id: str | int | bytes | memoryview = "*",
        maxlen=None,
        approximate: bool = True,
        nomkstream: bool = False,
        minid: Incomplete | None = None,
        limit: Incomplete | None = None,
    ) -> Any: ...
    def xautoclaim(
        self,
        name,
        groupname,
        consumername,
        min_idle_time,
        start_id: StreamIdT = "0-0",
        count: Incomplete | None = None,
        justid: bool = False,
    ) -> Any: ...
    def xclaim(
        self,
        name,
        groupname,
        consumername,
        min_idle_time,
        message_ids,
        idle=None,
        time=None,
        retrycount=None,
        force=False,
        justid=False,
    ) -> Any: ...
    def xdel(self, name, *ids) -> Any: ...
    def xgroup_create(self, name, groupname, id: str = "$", mkstream: bool = False, entries_read: int | None = None) -> Any: ...
    def xgroup_delconsumer(self, name, groupname, consumername) -> Any: ...
    def xgroup_destroy(self, name, groupname) -> Any: ...
    def xgroup_createconsumer(self, name, groupname, consumername) -> Any: ...
    def xgroup_setid(self, name, groupname, id, entries_read: int | None = None) -> Any: ...
    def xinfo_consumers(self, name, groupname) -> Any: ...
    def xinfo_groups(self, name) -> Any: ...
    def xinfo_stream(self, name, full: bool = False) -> Any: ...
    def xlen(self, name: _Key) -> Any: ...
    def xpending(self, name, groupname) -> Any: ...
    def xpending_range(
        self, name: _Key, groupname, min, max, count: int, consumername: Incomplete | None = None, idle: int | None = None
    ) -> Any: ...
    def xrange(self, name, min: str = "-", max: str = "+", count: Incomplete | None = None) -> Any: ...
    def xread(self, streams, count: Incomplete | None = None, block: Incomplete | None = None) -> Any: ...
    def xreadgroup(
        self,
        groupname,
        consumername,
        streams,
        count: Incomplete | None = None,
        block: Incomplete | None = None,
        noack: bool = False,
    ) -> Any: ...
    def xrevrange(self, name, max: str = "+", min: str = "-", count: Incomplete | None = None) -> Any: ...
    def xtrim(
        self, name, maxlen: int | None = None, approximate: bool = True, minid: Incomplete | None = None, limit: int | None = None
    ) -> Any: ...
    # endregion
    # region sorted set commands
    def zadd(
        self,
        name: _Key,
        mapping: Mapping[_Key, _Value],
        nx: bool = False,
        xx: bool = False,
        ch: bool = False,
        incr: bool = False,
        gt: Incomplete | None = False,
        lt: Incomplete | None = False,
    ) -> Any: ...
    def zcard(self, name: _Key) -> Any: ...
    def zcount(self, name: _Key, min: _Value, max: _Value) -> Any: ...
    def zdiff(self, keys, withscores: bool = False) -> Any: ...
    def zdiffstore(self, dest, keys) -> Any: ...
    def zincrby(self, name: _Key, amount: float, value: _Value) -> Any: ...
    def zinter(self, keys, aggregate: Incomplete | None = None, withscores: bool = False) -> Any: ...
    def zinterstore(self, dest: _Key, keys: Iterable[_Key], aggregate: Literal["SUM", "MIN", "MAX"] | None = None) -> Any: ...
    def zlexcount(self, name: _Key, min: _Value, max: _Value) -> Any: ...
    def zpopmax(self, name: _Key, count: int | None = None) -> Any: ...
    def zpopmin(self, name: _Key, count: int | None = None) -> Any: ...
    def zrandmember(self, key, count: Incomplete | None = None, withscores: bool = False) -> Any: ...
    @overload
    def bzpopmax(self, keys: _Key | Iterable[_Key], timeout: Literal[0] = 0) -> Any: ...
    @overload
    def bzpopmax(self, keys: _Key | Iterable[_Key], timeout: float) -> Any: ...
    @overload
    def bzpopmin(self, keys: _Key | Iterable[_Key], timeout: Literal[0] = 0) -> Any: ...
    @overload
    def bzpopmin(self, keys: _Key | Iterable[_Key], timeout: float) -> Any: ...
    @overload  # type: ignore[override]
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], Any],
        byscore: bool = False,
        bylex: bool = False,
        offset: int | None = None,
        num: int | None = None,
    ) -> Any: ...
    @overload
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], float] = ...,
        byscore: bool = False,
        bylex: bool = False,
        offset: int | None = None,
        num: int | None = None,
    ) -> Any: ...
    @overload
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], None],
        byscore: bool = False,
        bylex: bool = False,
        offset: int | None = None,
        num: int | None = None,
    ) -> Any: ...
    @overload
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], float] = ...,
        byscore: bool = False,
        bylex: bool = False,
        offset: int | None = None,
        num: int | None = None,
    ) -> Any: ...
    @overload
    def zrange(
        self,
        name: _Key,
        start: int,
        end: int,
        desc: bool = False,
        withscores: bool = False,
        score_cast_func: Callable[[_StrType], Any] = ...,
        byscore: bool = False,
        bylex: bool = False,
        offset: int | None = None,
        num: int | None = None,
    ) -> Any: ...
    @overload  # type: ignore[override]
    def zrevrange(
        self, name: _Key, start: int, end: int, withscores: Literal[True], score_cast_func: Callable[[_StrType], None]
    ) -> Any: ...
    @overload
    def zrevrange(self, name: _Key, start: int, end: int, withscores: Literal[True]) -> Any: ...
    @overload
    def zrevrange(
        self, name: _Key, start: int, end: int, withscores: bool = False, score_cast_func: Callable[[Any], Any] = ...
    ) -> Any: ...
    def zrangestore(
        self,
        dest,
        name,
        start,
        end,
        byscore: bool = False,
        bylex: bool = False,
        desc: bool = False,
        offset: Incomplete | None = None,
        num: Incomplete | None = None,
    ) -> Any: ...
    def zrangebylex(self, name: _Key, min: _Value, max: _Value, start: int | None = None, num: int | None = None) -> Any: ...
    def zrevrangebylex(self, name: _Key, max: _Value, min: _Value, start: int | None = None, num: int | None = None) -> Any: ...
    @overload  # type: ignore[override]
    def zrangebyscore(
        self,
        name: _Key,
        min: _Value,
        max: _Value,
        start: int | None = None,
        num: int | None = None,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], None],
    ) -> Any: ...
    @overload
    def zrangebyscore(
        self, name: _Key, min: _Value, max: _Value, start: int | None = None, num: int | None = None, *, withscores: Literal[True]
    ) -> Any: ...
    @overload
    def zrangebyscore(
        self,
        name: _Key,
        min: _Value,
        max: _Value,
        start: int | None = None,
        num: int | None = None,
        withscores: bool = False,
        score_cast_func: Callable[[_StrType], Any] = ...,
    ) -> Any: ...
    @overload
    def zrevrangebyscore(
        self,
        name: _Key,
        max: _Value,
        min: _Value,
        start: int | None = None,
        num: int | None = None,
        *,
        withscores: Literal[True],
        score_cast_func: Callable[[_StrType], Any],
    ) -> Any: ...
    @overload
    def zrevrangebyscore(
        self, name: _Key, max: _Value, min: _Value, start: int | None = None, num: int | None = None, *, withscores: Literal[True]
    ) -> Any: ...
    @overload
    def zrevrangebyscore(
        self,
        name: _Key,
        max: _Value,
        min: _Value,
        start: int | None = None,
        num: int | None = None,
        withscores: bool = False,
        score_cast_func: Callable[[_StrType], Any] = ...,
    ) -> Any: ...
    def zrank(self, name: _Key, value: _Value, withscore: bool = False) -> Any: ...
    def zrem(self, name: _Key, *values: _Value) -> Any: ...
    def zremrangebylex(self, name: _Key, min: _Value, max: _Value) -> Any: ...
    def zremrangebyrank(self, name: _Key, min: int, max: int) -> Any: ...
    def zremrangebyscore(self, name: _Key, min: _Value, max: _Value) -> Any: ...
    def zrevrank(self, name: _Key, value: _Value, withscore: bool = False) -> Any: ...
    def zscore(self, name: _Key, value: _Value) -> Any: ...
    def zunion(self, keys, aggregate: Incomplete | None = None, withscores: bool = False) -> Any: ...
    def zunionstore(self, dest: _Key, keys: Iterable[_Key], aggregate: Literal["SUM", "MIN", "MAX"] | None = None) -> Any: ...
    def zmscore(self, key, members) -> Any: ...
    # endregion
    # region management commands
    def bgrewriteaof(self, **kwargs: _CommandOptions) -> Any: ...
    def bgsave(self, schedule: bool = True, **kwargs: _CommandOptions) -> Any: ...
    def role(self) -> Any: ...
    def client_kill(self, address: str, **kwargs: _CommandOptions) -> Any: ...
    def client_kill_filter(
        self,
        _id: Incomplete | None = None,
        _type: Incomplete | None = None,
        addr: Incomplete | None = None,
        skipme: Incomplete | None = None,
        laddr: Incomplete | None = None,
        user: Incomplete | None = None,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def client_info(self, **kwargs: _CommandOptions) -> Any: ...
    def client_list(self, _type: str | None = None, client_id: list[str] = [], **kwargs: _CommandOptions) -> Any: ...
    def client_getname(self, **kwargs: _CommandOptions) -> Any: ...
    def client_getredir(self, **kwargs: _CommandOptions) -> Any: ...
    def client_reply(self, reply, **kwargs: _CommandOptions) -> Any: ...
    def client_id(self, **kwargs: _CommandOptions) -> Any: ...
    def client_tracking_on(
        self,
        clientid: Incomplete | None = None,
        prefix=[],
        bcast: bool = False,
        optin: bool = False,
        optout: bool = False,
        noloop: bool = False,
    ) -> Any: ...
    def client_tracking_off(
        self,
        clientid: Incomplete | None = None,
        prefix=[],
        bcast: bool = False,
        optin: bool = False,
        optout: bool = False,
        noloop: bool = False,
    ) -> Any: ...
    def client_tracking(
        self,
        on: bool = True,
        clientid: Incomplete | None = None,
        prefix=[],
        bcast: bool = False,
        optin: bool = False,
        optout: bool = False,
        noloop: bool = False,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def client_trackinginfo(self, **kwargs: _CommandOptions) -> Any: ...
    def client_setname(self, name: str, **kwargs: _CommandOptions) -> Any: ...
    def client_unblock(self, client_id, error: bool = False, **kwargs: _CommandOptions) -> Any: ...
    def client_pause(self, timeout, all: bool = True, **kwargs: _CommandOptions) -> Any: ...
    def client_unpause(self, **kwargs: _CommandOptions) -> Any: ...
    def command(self, **kwargs: _CommandOptions) -> Any: ...
    def command_info(self, **kwargs: _CommandOptions) -> Any: ...
    def command_count(self, **kwargs: _CommandOptions) -> Any: ...
    def config_get(self, pattern: PatternT = "*", *args: PatternT, **kwargs: _CommandOptions) -> Any: ...
    def config_set(self, name: KeyT, value: EncodableT, *args: KeyT | EncodableT, **kwargs: _CommandOptions) -> Any: ...
    def config_resetstat(self, **kwargs: _CommandOptions) -> Any: ...
    def config_rewrite(self, **kwargs: _CommandOptions) -> Any: ...
    def dbsize(self, **kwargs: _CommandOptions) -> Any: ...
    def debug_object(self, key, **kwargs: _CommandOptions) -> Any: ...
    def debug_segfault(self, **kwargs: _CommandOptions) -> Any: ...
    def echo(self, value: _Value, **kwargs: _CommandOptions) -> Any: ...
    def flushall(self, asynchronous: bool = False, **kwargs: _CommandOptions) -> Any: ...
    def flushdb(self, asynchronous: bool = False, **kwargs: _CommandOptions) -> Any: ...
    def sync(self) -> Any: ...
    def psync(self, replicationid, offset) -> Any: ...
    def swapdb(self, first, second, **kwargs: _CommandOptions) -> Any: ...
    def select(self, index, **kwargs: _CommandOptions) -> Any: ...
    def info(self, section: _Key | None = None, *args: _Key, **kwargs: _CommandOptions) -> Any: ...
    def lastsave(self, **kwargs: _CommandOptions) -> Any: ...
    def lolwut(self, *version_numbers: _Value, **kwargs: _CommandOptions) -> Any: ...
    def migrate(
        self,
        host,
        port,
        keys,
        destination_db,
        timeout,
        copy: bool = False,
        replace: bool = False,
        auth: Incomplete | None = None,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def object(self, infotype, key, **kwargs: _CommandOptions) -> Any: ...
    def memory_doctor(self, **kwargs: _CommandOptions) -> Any: ...
    def memory_help(self, **kwargs: _CommandOptions) -> Any: ...
    def memory_stats(self, **kwargs: _CommandOptions) -> Any: ...
    def memory_malloc_stats(self, **kwargs: _CommandOptions) -> Any: ...
    def memory_usage(self, key, samples: Incomplete | None = None, **kwargs: _CommandOptions) -> Any: ...
    def memory_purge(self, **kwargs: _CommandOptions) -> Any: ...
    def ping(self, **kwargs: _CommandOptions) -> Any: ...
    def quit(self, **kwargs: _CommandOptions) -> Any: ...
    def replicaof(self, *args, **kwargs: _CommandOptions) -> Any: ...
    def save(self, **kwargs: _CommandOptions) -> Any: ...
    def shutdown(
        self,
        save: bool = False,
        nosave: bool = False,
        now: bool = False,
        force: bool = False,
        abort: bool = False,
        **kwargs: _CommandOptions,
    ) -> Any: ...
    def slaveof(self, host: Incomplete | None = None, port: Incomplete | None = None, **kwargs: _CommandOptions) -> Any: ...
    def slowlog_get(self, num: Incomplete | None = None, **kwargs: _CommandOptions) -> Any: ...
    def slowlog_len(self, **kwargs: _CommandOptions) -> Any: ...
    def slowlog_reset(self, **kwargs: _CommandOptions) -> Any: ...
    def time(self, **kwargs: _CommandOptions) -> Any: ...
    def wait(self, num_replicas, timeout, **kwargs: _CommandOptions) -> Any: ...
    # endregion
    # region module commands
    def module_load(self, path, *args) -> Any: ...
    def module_unload(self, name) -> Any: ...
    def module_list(self) -> Any: ...
    def command_getkeys(self, *args) -> Any: ...
    # endregion
    # region pubsub commands
    def publish(self, channel: _Key, message: _Key, **kwargs: _CommandOptions) -> Any: ...
    def pubsub_channels(self, pattern: _Key = "*", **kwargs: _CommandOptions) -> Any: ...
    def pubsub_numpat(self, **kwargs: _CommandOptions) -> Any: ...
    def pubsub_numsub(self, *args: _Key, **kwargs: _CommandOptions) -> Any: ...
    # endregion
    # region script commands
    def eval(self, script, numkeys, *keys_and_args) -> Any: ...
    def evalsha(self, sha, numkeys, *keys_and_args) -> Any: ...
    def script_exists(self, *args) -> Any: ...
    def script_debug(self, *args) -> Any: ...
    def script_flush(self, sync_type: Incomplete | None = None) -> Any: ...
    def script_kill(self) -> Any: ...
    def script_load(self, script) -> Any: ...
    def register_script(self, script: str | _StrType) -> Any: ...  # type: ignore[override]
    # endregion



from _typeshed import Incomplete, OpenBinaryMode
from asyncio import AbstractEventLoop
from collections.abc import Generator, Iterable
from tempfile import TemporaryDirectory
from typing import TypeVar

from aiofiles.base import AsyncBase as AsyncBase
from aiofiles.threadpool.utils import (
    cond_delegate_to_executor as cond_delegate_to_executor,
    delegate_to_executor as delegate_to_executor,
    proxy_property_directly as proxy_property_directly,
)

_T = TypeVar("_T")

class AsyncSpooledTemporaryFile(AsyncBase[_T]):
    def fileno(self) -> Generator[Incomplete, Incomplete, Incomplete]: ...
    def rollover(self) -> Generator[Incomplete, Incomplete, Incomplete]: ...
    async def close(self) -> None: ...
    async def flush(self) -> None: ...
    async def isatty(self) -> bool: ...
    # All must return `AnyStr`:
    async def read(self, n: int = ..., /): ...
    async def readline(self, limit: int | None = ..., /): ...
    async def readlines(self, hint: int = ..., /) -> list[Incomplete]: ...
    # ---
    async def seek(self, offset: int, whence: int = ...) -> int: ...
    async def tell(self) -> int: ...
    async def truncate(self, size: int | None = ...) -> None: ...
    @property
    def closed(self) -> bool: ...
    @property
    def encoding(self) -> str: ...
    @property
    def mode(self) -> OpenBinaryMode: ...
    @property
    def name(self) -> str: ...
    @property
    def newlines(self) -> str: ...
    # Both should work with `AnyStr`, like in `tempfile`:
    async def write(self, s) -> int: ...
    async def writelines(self, iterable: Iterable[Incomplete]) -> None: ...

class AsyncTemporaryDirectory:
    async def cleanup(self) -> None: ...
    @property
    def name(self): ...  # should be `AnyStr`
    def __init__(
        self, file: TemporaryDirectory[Incomplete], loop: AbstractEventLoop | None, executor: Incomplete | None
    ) -> None: ...
    async def close(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def graph_edit_distance(
    G1,
    G2,
    node_match: Incomplete | None = None,
    edge_match: Incomplete | None = None,
    node_subst_cost: Incomplete | None = None,
    node_del_cost: Incomplete | None = None,
    node_ins_cost: Incomplete | None = None,
    edge_subst_cost: Incomplete | None = None,
    edge_del_cost: Incomplete | None = None,
    edge_ins_cost: Incomplete | None = None,
    roots: Incomplete | None = None,
    upper_bound: Incomplete | None = None,
    timeout: Incomplete | None = None,
): ...
@_dispatch
def optimal_edit_paths(
    G1,
    G2,
    node_match: Incomplete | None = None,
    edge_match: Incomplete | None = None,
    node_subst_cost: Incomplete | None = None,
    node_del_cost: Incomplete | None = None,
    node_ins_cost: Incomplete | None = None,
    edge_subst_cost: Incomplete | None = None,
    edge_del_cost: Incomplete | None = None,
    edge_ins_cost: Incomplete | None = None,
    upper_bound: Incomplete | None = None,
): ...
@_dispatch
def optimize_graph_edit_distance(
    G1,
    G2,
    node_match: Incomplete | None = None,
    edge_match: Incomplete | None = None,
    node_subst_cost: Incomplete | None = None,
    node_del_cost: Incomplete | None = None,
    node_ins_cost: Incomplete | None = None,
    edge_subst_cost: Incomplete | None = None,
    edge_del_cost: Incomplete | None = None,
    edge_ins_cost: Incomplete | None = None,
    upper_bound: Incomplete | None = None,
) -> Generator[Incomplete, None, None]: ...
@_dispatch
def optimize_edit_paths(
    G1,
    G2,
    node_match: Incomplete | None = None,
    edge_match: Incomplete | None = None,
    node_subst_cost: Incomplete | None = None,
    node_del_cost: Incomplete | None = None,
    node_ins_cost: Incomplete | None = None,
    edge_subst_cost: Incomplete | None = None,
    edge_del_cost: Incomplete | None = None,
    edge_ins_cost: Incomplete | None = None,
    upper_bound: Incomplete | None = None,
    strictly_decreasing: bool = True,
    roots: Incomplete | None = None,
    timeout: Incomplete | None = None,
) -> Generator[Incomplete, None, Incomplete]: ...
@_dispatch
def simrank_similarity(
    G,
    source: Incomplete | None = None,
    target: Incomplete | None = None,
    importance_factor: float = 0.9,
    max_iterations: int = 1000,
    tolerance: float = 0.0001,
): ...
@_dispatch
def panther_similarity(
    G, source, k: int = 5, path_length: int = 5, c: float = 0.5, delta: float = 0.1, eps: Incomplete | None = None
): ...
@_dispatch
def generate_random_paths(
    G, sample_size, path_length: int = 5, index_map: Incomplete | None = None
) -> Generator[Incomplete, None, None]: ...



import sys
from collections.abc import Callable, Iterable, Iterator, Sequence
from typing import Any, AnyStr, Generic, Literal, NamedTuple, TypeVar, overload

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "get_close_matches",
    "ndiff",
    "restore",
    "SequenceMatcher",
    "Differ",
    "IS_CHARACTER_JUNK",
    "IS_LINE_JUNK",
    "context_diff",
    "unified_diff",
    "diff_bytes",
    "HtmlDiff",
    "Match",
]

_T = TypeVar("_T")

class Match(NamedTuple):
    a: int
    b: int
    size: int

class SequenceMatcher(Generic[_T]):
    @overload
    def __init__(self, isjunk: Callable[[_T], bool] | None, a: Sequence[_T], b: Sequence[_T], autojunk: bool = True) -> None: ...
    @overload
    def __init__(self, *, a: Sequence[_T], b: Sequence[_T], autojunk: bool = True) -> None: ...
    @overload
    def __init__(
        self: SequenceMatcher[str],
        isjunk: Callable[[str], bool] | None = None,
        a: Sequence[str] = "",
        b: Sequence[str] = "",
        autojunk: bool = True,
    ) -> None: ...
    def set_seqs(self, a: Sequence[_T], b: Sequence[_T]) -> None: ...
    def set_seq1(self, a: Sequence[_T]) -> None: ...
    def set_seq2(self, b: Sequence[_T]) -> None: ...
    if sys.version_info >= (3, 9):
        def find_longest_match(self, alo: int = 0, ahi: int | None = None, blo: int = 0, bhi: int | None = None) -> Match: ...
    else:
        def find_longest_match(self, alo: int, ahi: int, blo: int, bhi: int) -> Match: ...

    def get_matching_blocks(self) -> list[Match]: ...
    def get_opcodes(self) -> list[tuple[Literal["replace", "delete", "insert", "equal"], int, int, int, int]]: ...
    def get_grouped_opcodes(self, n: int = 3) -> Iterable[list[tuple[str, int, int, int, int]]]: ...
    def ratio(self) -> float: ...
    def quick_ratio(self) -> float: ...
    def real_quick_ratio(self) -> float: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

@overload
def get_close_matches(word: AnyStr, possibilities: Iterable[AnyStr], n: int = 3, cutoff: float = 0.6) -> list[AnyStr]: ...
@overload
def get_close_matches(
    word: Sequence[_T], possibilities: Iterable[Sequence[_T]], n: int = 3, cutoff: float = 0.6
) -> list[Sequence[_T]]: ...

class Differ:
    def __init__(self, linejunk: Callable[[str], bool] | None = None, charjunk: Callable[[str], bool] | None = None) -> None: ...
    def compare(self, a: Sequence[str], b: Sequence[str]) -> Iterator[str]: ...

def IS_LINE_JUNK(line: str, pat: Any = ...) -> bool: ...  # pat is undocumented
def IS_CHARACTER_JUNK(ch: str, ws: str = " \t") -> bool: ...  # ws is undocumented
def unified_diff(
    a: Sequence[str],
    b: Sequence[str],
    fromfile: str = "",
    tofile: str = "",
    fromfiledate: str = "",
    tofiledate: str = "",
    n: int = 3,
    lineterm: str = "\n",
) -> Iterator[str]: ...
def context_diff(
    a: Sequence[str],
    b: Sequence[str],
    fromfile: str = "",
    tofile: str = "",
    fromfiledate: str = "",
    tofiledate: str = "",
    n: int = 3,
    lineterm: str = "\n",
) -> Iterator[str]: ...
def ndiff(
    a: Sequence[str],
    b: Sequence[str],
    linejunk: Callable[[str], bool] | None = None,
    charjunk: Callable[[str], bool] | None = ...,
) -> Iterator[str]: ...

class HtmlDiff:
    def __init__(
        self,
        tabsize: int = 8,
        wrapcolumn: int | None = None,
        linejunk: Callable[[str], bool] | None = None,
        charjunk: Callable[[str], bool] | None = ...,
    ) -> None: ...
    def make_file(
        self,
        fromlines: Sequence[str],
        tolines: Sequence[str],
        fromdesc: str = "",
        todesc: str = "",
        context: bool = False,
        numlines: int = 5,
        *,
        charset: str = "utf-8",
    ) -> str: ...
    def make_table(
        self,
        fromlines: Sequence[str],
        tolines: Sequence[str],
        fromdesc: str = "",
        todesc: str = "",
        context: bool = False,
        numlines: int = 5,
    ) -> str: ...

def restore(delta: Iterable[str], which: int) -> Iterator[str]: ...
def diff_bytes(
    dfunc: Callable[[Sequence[str], Sequence[str], str, str, str, str, int, str], Iterator[str]],
    a: Iterable[bytes | bytearray],
    b: Iterable[bytes | bytearray],
    fromfile: bytes | bytearray = b"",
    tofile: bytes | bytearray = b"",
    fromfiledate: bytes | bytearray = b"",
    tofiledate: bytes | bytearray = b"",
    n: int = 3,
    lineterm: bytes | bytearray = b"\n",
) -> Iterator[bytes]: ...



from collections.abc import Callable, Iterable, Sequence
from re import Pattern
from types import FunctionType, MethodType
from typing import Any, Literal, TypedDict, overload
from typing_extensions import TypeAlias

from Xlib import error
from Xlib._typing import ErrorHandler
from Xlib.protocol import display, request, rq
from Xlib.xobject import colormap, cursor, drawable, fontable, resource

_ResourceBaseClass: TypeAlias = (
    resource.Resource
    | drawable.Drawable
    | drawable.Window
    | drawable.Pixmap
    | fontable.Fontable
    | fontable.Font
    | fontable.GC
    | colormap.Colormap
    | cursor.Cursor
)

# Is the type of the `_resource_baseclasses` variable, defined in this file at runtime
class _ResourceBaseClassesType(TypedDict):  # noqa: Y049
    resource: type[resource.Resource]
    drawable: type[drawable.Drawable]
    window: type[drawable.Window]
    pixmap: type[drawable.Pixmap]
    fontable: type[fontable.Fontable]
    font: type[fontable.Font]
    gc: type[fontable.GC]
    colormap: type[colormap.Colormap]
    cursor: type[cursor.Cursor]

class _BaseDisplay(display.Display):
    def __init__(self, display: str | None = None) -> None: ...
    def get_atom(self, atomname: str, only_if_exists: bool = False) -> int: ...

class Display:
    display: _BaseDisplay
    keysym_translations: dict[int, str]
    extensions: list[str]
    class_extension_dicts: dict[str, dict[str, FunctionType]]
    display_extension_methods: dict[str, Callable[..., Any]]
    extension_event: rq.DictWrapper
    def __init__(self, display: str | None = None) -> None: ...
    def get_display_name(self) -> str: ...
    def fileno(self) -> int: ...
    def close(self) -> None: ...
    def set_error_handler(self, handler: ErrorHandler[object] | None) -> None: ...
    def flush(self) -> None: ...
    def sync(self) -> None: ...
    def next_event(self) -> rq.Event: ...
    def pending_events(self) -> int: ...
    def has_extension(self, extension: str) -> bool: ...
    @overload
    def create_resource_object(self, type: Literal["resource"], id: int) -> resource.Resource: ...
    @overload
    def create_resource_object(self, type: Literal["drawable"], id: int) -> drawable.Drawable: ...
    @overload
    def create_resource_object(self, type: Literal["window"], id: int) -> drawable.Window: ...
    @overload
    def create_resource_object(self, type: Literal["pixmap"], id: int) -> drawable.Pixmap: ...
    @overload
    def create_resource_object(self, type: Literal["fontable"], id: int) -> fontable.Fontable: ...
    @overload
    def create_resource_object(self, type: Literal["font"], id: int) -> fontable.Font: ...
    @overload
    def create_resource_object(self, type: Literal["gc"], id: int) -> fontable.GC: ...
    @overload
    def create_resource_object(self, type: Literal["colormap"], id: int) -> colormap.Colormap: ...
    @overload
    def create_resource_object(self, type: Literal["cursor"], id: int) -> cursor.Cursor: ...
    @overload
    def create_resource_object(self, type: str, id: int) -> resource.Resource: ...
    def __getattr__(self, attr: str) -> MethodType: ...
    def screen(self, sno: int | None = None) -> rq.Struct: ...
    def screen_count(self) -> int: ...
    def get_default_screen(self) -> int: ...
    def extension_add_method(self, object: str, name: str, function: Callable[..., Any]) -> None: ...
    def extension_add_event(self, code: int, evt: type, name: str | None = None) -> None: ...
    def extension_add_subevent(self, code: int, subcode: int | None, evt: type[rq.Event], name: str | None = None) -> None: ...
    def extension_add_error(self, code: int, err: type[error.XError]) -> None: ...
    def keycode_to_keysym(self, keycode: int, index: int) -> int: ...
    def keysym_to_keycode(self, keysym: int) -> int: ...
    def keysym_to_keycodes(self, keysym: int) -> Iterable[tuple[int, int]]: ...
    def refresh_keyboard_mapping(self, evt: rq.Event) -> None: ...
    def lookup_string(self, keysym: int) -> str | None: ...
    def rebind_string(self, keysym: int, newstring: str | None) -> None: ...
    def intern_atom(self, name: str, only_if_exists: bool = False) -> int: ...
    def get_atom(self, atom: str, only_if_exists: bool = False) -> int: ...
    def get_atom_name(self, atom: int) -> str: ...
    def get_selection_owner(self, selection: int) -> int: ...
    def send_event(
        self,
        destination: int,
        event: rq.Event,
        event_mask: int = 0,
        propagate: bool = False,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def ungrab_pointer(self, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def change_active_pointer_grab(
        self, event_mask: int, cursor: cursor.Cursor, time: int, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def ungrab_keyboard(self, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def allow_events(self, mode: int, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def grab_server(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def ungrab_server(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def warp_pointer(
        self,
        x: int,
        y: int,
        src_window: int = 0,
        src_x: int = 0,
        src_y: int = 0,
        src_width: int = 0,
        src_height: int = 0,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def set_input_focus(self, focus: int, revert_to: int, time: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_input_focus(self) -> request.GetInputFocus: ...
    def query_keymap(self) -> bytes: ...  # TODO: Validate if this is correct
    def open_font(self, name: str) -> _ResourceBaseClass | None: ...
    def list_fonts(self, pattern: Pattern[str] | str, max_names: int) -> list[str]: ...
    def list_fonts_with_info(self, pattern: Pattern[str] | str, max_names: int) -> request.ListFontsWithInfo: ...
    def set_font_path(self, path: Sequence[str], onerror: ErrorHandler[object] | None = None) -> None: ...
    def get_font_path(self) -> list[str]: ...
    def query_extension(self, name: str) -> request.QueryExtension | None: ...
    def list_extensions(self) -> list[str]: ...
    def change_keyboard_mapping(
        self, first_keycode: int, keysyms: Sequence[Sequence[int]], onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def get_keyboard_mapping(self, first_keycode: int, count: int) -> list[tuple[int, ...]]: ...
    def change_keyboard_control(self, onerror: ErrorHandler[object] | None = None, **keys: object) -> None: ...
    def get_keyboard_control(self) -> request.GetKeyboardControl: ...
    def bell(self, percent: int = 0, onerror: ErrorHandler[object] | None = None) -> None: ...
    def change_pointer_control(
        self, accel: tuple[int, int] | None = None, threshold: int | None = None, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def get_pointer_control(self) -> request.GetPointerControl: ...
    def set_screen_saver(
        self, timeout: int, interval: int, prefer_blank: int, allow_exposures: int, onerror: ErrorHandler[object] | None = None
    ) -> None: ...
    def get_screen_saver(self) -> request.GetScreenSaver: ...
    def change_hosts(
        self,
        mode: int,
        host_family: int,
        host: Sequence[int] | Sequence[bytes],  # TODO: validate
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def list_hosts(self) -> request.ListHosts: ...
    def set_access_control(self, mode: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def set_close_down_mode(self, mode: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def force_screen_saver(self, mode: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def set_pointer_mapping(self, map: Sequence[int]) -> int: ...
    def get_pointer_mapping(self) -> list[int]: ...
    def set_modifier_mapping(self, keycodes: rq._ModifierMappingList8Elements) -> int: ...
    def get_modifier_mapping(self) -> Sequence[Sequence[int]]: ...
    def no_operation(self, onerror: ErrorHandler[object] | None = None) -> None: ...



from types import TracebackType
from typing import Generic, Literal, Protocol, TypeVar, overload
from typing_extensions import TypeAlias

from gevent._abstract_linkable import AbstractLinkable

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
# gevent generally allows the tracebock to be omitted, it can also fail to serialize
# in which case it will be None as well.
_ExcInfo: TypeAlias = tuple[type[BaseException], BaseException, TracebackType | None]
_OptExcInfo: TypeAlias = _ExcInfo | tuple[None, None, None]

class _ValueSource(Protocol[_T_co]):
    def successful(self) -> bool: ...
    @property
    def value(self) -> _T_co | None: ...
    @property
    def exception(self) -> BaseException | None: ...

class Event(AbstractLinkable):
    def __init__(self) -> None: ...
    def is_set(self) -> bool: ...
    def isSet(self) -> bool: ...
    def ready(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    @overload
    def wait(self, timeout: None = None) -> Literal[True]: ...
    @overload
    def wait(self, timeout: float) -> bool: ...

class AsyncResult(AbstractLinkable, Generic[_T]):
    def __init__(self) -> None: ...
    @property
    def value(self) -> _T | None: ...
    @property
    def exc_info(self) -> _OptExcInfo | tuple[None, None, None] | tuple[()]: ...
    @property
    def exception(self) -> BaseException | None: ...
    def ready(self) -> bool: ...
    def successful(self) -> bool: ...
    def set(self, value: _T | None = None) -> None: ...
    @overload
    def set_exception(self, exception: BaseException, exc_info: None = None) -> None: ...
    @overload
    def set_exception(self, exception: BaseException | None, exc_info: _OptExcInfo) -> None: ...
    # technically get/get_nowait/result should just return _T, but the API is designed in
    # such a way that it is perfectly legal for a ValueSource to have neither its value nor
    # its exception set, while still being marked successful, at which point None would be
    # stored into value, it's also legal to call set without arguments, which has the same
    # effect, this is a little annoying, since it will introduce some additional None checks
    # that may not be necessary, but it's impossible to annotate this situation, so for now
    # we just deal with the possibly redundant None checks...
    def get(self, block: bool = True, timeout: float | None = None) -> _T | None: ...
    def get_nowait(self) -> _T | None: ...
    def wait(self, timeout: float | None = None) -> _T | None: ...
    def __call__(self, source: _ValueSource[_T]) -> None: ...
    def result(self, timeout: float | None = None) -> _T | None: ...
    set_result = set
    def done(self) -> bool: ...
    def cancel(self) -> Literal[False]: ...
    def cancelled(self) -> Literal[False]: ...



from _typeshed import Incomplete

from influxdb_client.client.exceptions import InfluxDBError

class ApiException(InfluxDBError):
    status: Incomplete
    reason: Incomplete
    body: Incomplete
    headers: Incomplete
    def __init__(
        self, status: Incomplete | None = None, reason: Incomplete | None = None, http_resp: Incomplete | None = None
    ) -> None: ...

class _BaseRESTClient:
    logger: Incomplete
    @staticmethod
    def log_request(method: str, url: str): ...
    @staticmethod
    def log_response(status: str): ...
    @staticmethod
    def log_body(body: object, prefix: str): ...
    @staticmethod
    def log_headers(headers: dict[str, str], prefix: str): ...



from _typeshed import Incomplete

class StackAssociations:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, kind: Incomplete | None = None, meta_name: Incomplete | None = None) -> None: ...
    @property
    def kind(self): ...
    @kind.setter
    def kind(self, kind) -> None: ...
    @property
    def meta_name(self): ...
    @meta_name.setter
    def meta_name(self, meta_name) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

from influxdb_client.domain.authorization_update_request import AuthorizationUpdateRequest

class AuthorizationPostRequest(AuthorizationUpdateRequest):
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        org_id: Incomplete | None = None,
        user_id: Incomplete | None = None,
        permissions: Incomplete | None = None,
        status: str = "active",
        description: Incomplete | None = None,
    ) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def user_id(self): ...
    @user_id.setter
    def user_id(self, user_id) -> None: ...
    @property
    def permissions(self): ...
    @permissions.setter
    def permissions(self, permissions) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from networkx.algorithms import (
    approximation as approximation,
    assortativity as assortativity,
    bipartite as bipartite,
    centrality as centrality,
    chordal as chordal,
    clique as clique,
    cluster as cluster,
    coloring as coloring,
    community as community,
    components as components,
    connectivity as connectivity,
    flow as flow,
    isomorphism as isomorphism,
    link_analysis as link_analysis,
    lowest_common_ancestors as lowest_common_ancestors,
    node_classification as node_classification,
    operators as operators,
    shortest_paths as shortest_paths,
    tournament as tournament,
    traversal as traversal,
    tree as tree,
)
from networkx.algorithms.assortativity import *
from networkx.algorithms.asteroidal import *
from networkx.algorithms.bipartite import (
    complete_bipartite_graph as complete_bipartite_graph,
    is_bipartite as is_bipartite,
    projected_graph as projected_graph,
)
from networkx.algorithms.boundary import *
from networkx.algorithms.bridges import *
from networkx.algorithms.centrality import *
from networkx.algorithms.chains import *
from networkx.algorithms.chordal import *
from networkx.algorithms.clique import *
from networkx.algorithms.cluster import *
from networkx.algorithms.coloring import *
from networkx.algorithms.communicability_alg import *
from networkx.algorithms.components import *
from networkx.algorithms.connectivity import (
    all_node_cuts as all_node_cuts,
    all_pairs_node_connectivity as all_pairs_node_connectivity,
    average_node_connectivity as average_node_connectivity,
    edge_connectivity as edge_connectivity,
    edge_disjoint_paths as edge_disjoint_paths,
    is_k_edge_connected as is_k_edge_connected,
    k_components as k_components,
    k_edge_augmentation as k_edge_augmentation,
    k_edge_components as k_edge_components,
    k_edge_subgraphs as k_edge_subgraphs,
    minimum_edge_cut as minimum_edge_cut,
    minimum_node_cut as minimum_node_cut,
    node_connectivity as node_connectivity,
    node_disjoint_paths as node_disjoint_paths,
    stoer_wagner as stoer_wagner,
)
from networkx.algorithms.core import *
from networkx.algorithms.covering import *
from networkx.algorithms.cuts import *
from networkx.algorithms.cycles import *
from networkx.algorithms.d_separation import *
from networkx.algorithms.dag import *
from networkx.algorithms.distance_measures import *
from networkx.algorithms.distance_regular import *
from networkx.algorithms.dominance import *
from networkx.algorithms.dominating import *
from networkx.algorithms.efficiency_measures import *
from networkx.algorithms.euler import *
from networkx.algorithms.flow import (
    capacity_scaling as capacity_scaling,
    cost_of_flow as cost_of_flow,
    gomory_hu_tree as gomory_hu_tree,
    max_flow_min_cost as max_flow_min_cost,
    maximum_flow as maximum_flow,
    maximum_flow_value as maximum_flow_value,
    min_cost_flow as min_cost_flow,
    min_cost_flow_cost as min_cost_flow_cost,
    minimum_cut as minimum_cut,
    minimum_cut_value as minimum_cut_value,
    network_simplex as network_simplex,
)
from networkx.algorithms.graph_hashing import *
from networkx.algorithms.graphical import *
from networkx.algorithms.hierarchy import *
from networkx.algorithms.hybrid import *
from networkx.algorithms.isolate import *
from networkx.algorithms.isomorphism import (
    could_be_isomorphic as could_be_isomorphic,
    fast_could_be_isomorphic as fast_could_be_isomorphic,
    faster_could_be_isomorphic as faster_could_be_isomorphic,
    is_isomorphic as is_isomorphic,
)
from networkx.algorithms.isomorphism.vf2pp import *
from networkx.algorithms.link_analysis import *
from networkx.algorithms.link_prediction import *
from networkx.algorithms.lowest_common_ancestors import *
from networkx.algorithms.matching import *
from networkx.algorithms.minors import *
from networkx.algorithms.mis import *
from networkx.algorithms.moral import *
from networkx.algorithms.non_randomness import *
from networkx.algorithms.operators import *
from networkx.algorithms.planar_drawing import *
from networkx.algorithms.planarity import *
from networkx.algorithms.polynomials import *
from networkx.algorithms.reciprocity import *
from networkx.algorithms.regular import *
from networkx.algorithms.richclub import *
from networkx.algorithms.shortest_paths import *
from networkx.algorithms.similarity import *
from networkx.algorithms.simple_paths import *
from networkx.algorithms.smallworld import *
from networkx.algorithms.smetric import *
from networkx.algorithms.sparsifiers import *
from networkx.algorithms.structuralholes import *
from networkx.algorithms.summarization import *
from networkx.algorithms.swap import *
from networkx.algorithms.traversal import *
from networkx.algorithms.tree.branchings import (
    ArborescenceIterator as ArborescenceIterator,
    maximum_branching as maximum_branching,
    maximum_spanning_arborescence as maximum_spanning_arborescence,
    minimum_branching as minimum_branching,
    minimum_spanning_arborescence as minimum_spanning_arborescence,
)
from networkx.algorithms.tree.coding import *
from networkx.algorithms.tree.decomposition import *
from networkx.algorithms.tree.mst import *
from networkx.algorithms.tree.operations import *
from networkx.algorithms.tree.recognition import *
from networkx.algorithms.triads import *
from networkx.algorithms.vitality import *
from networkx.algorithms.voronoi import *
from networkx.algorithms.wiener import *



from _typeshed import FileDescriptorOrPath, Incomplete, SupportsWrite
from collections.abc import Iterable, Mapping
from distutils.cmd import Command
from re import Pattern
from typing import IO, Any, ClassVar, TypeVar, overload
from typing_extensions import TypeAlias

command_re: Pattern[str]

_OptionsList: TypeAlias = list[tuple[str, str | None, str, int] | tuple[str, str | None, str]]
_CommandT = TypeVar("_CommandT", bound=Command)

class DistributionMetadata:
    def __init__(self, path: FileDescriptorOrPath | None = None) -> None: ...
    name: str | None
    version: str | None
    author: str | None
    author_email: str | None
    maintainer: str | None
    maintainer_email: str | None
    url: str | None
    license: str | None
    description: str | None
    long_description: str | None
    keywords: str | list[str] | None
    platforms: str | list[str] | None
    classifiers: str | list[str] | None
    download_url: str | None
    provides: list[str] | None
    requires: list[str] | None
    obsoletes: list[str] | None
    def read_pkg_file(self, file: IO[str]) -> None: ...
    def write_pkg_info(self, base_dir: str) -> None: ...
    def write_pkg_file(self, file: SupportsWrite[str]) -> None: ...
    def get_name(self) -> str: ...
    def get_version(self) -> str: ...
    def get_fullname(self) -> str: ...
    def get_author(self) -> str: ...
    def get_author_email(self) -> str: ...
    def get_maintainer(self) -> str: ...
    def get_maintainer_email(self) -> str: ...
    def get_contact(self) -> str: ...
    def get_contact_email(self) -> str: ...
    def get_url(self) -> str: ...
    def get_license(self) -> str: ...
    def get_licence(self) -> str: ...
    def get_description(self) -> str: ...
    def get_long_description(self) -> str: ...
    def get_keywords(self) -> str | list[str]: ...
    def get_platforms(self) -> str | list[str]: ...
    def get_classifiers(self) -> str | list[str]: ...
    def get_download_url(self) -> str: ...
    def get_requires(self) -> list[str]: ...
    def set_requires(self, value: Iterable[str]) -> None: ...
    def get_provides(self) -> list[str]: ...
    def set_provides(self, value: Iterable[str]) -> None: ...
    def get_obsoletes(self) -> list[str]: ...
    def set_obsoletes(self, value: Iterable[str]) -> None: ...

class Distribution:
    cmdclass: dict[str, type[Command]]
    metadata: DistributionMetadata
    def __init__(self, attrs: Mapping[str, Any] | None = None) -> None: ...
    def get_option_dict(self, command: str) -> dict[str, tuple[str, str]]: ...
    def parse_config_files(self, filenames: Iterable[str] | None = None) -> None: ...
    def get_command_obj(self, command: str, create: bool = True) -> Command | None: ...
    global_options: ClassVar[_OptionsList]
    common_usage: ClassVar[str]
    display_options: ClassVar[_OptionsList]
    display_option_names: ClassVar[list[str]]
    negative_opt: ClassVar[dict[str, str]]
    verbose: int
    dry_run: int
    help: int
    command_packages: list[str] | None
    script_name: str | None
    script_args: list[str] | None
    command_options: dict[str, dict[str, tuple[str, str]]]
    dist_files: list[tuple[str, str, str]]
    packages: Incomplete
    package_data: dict[str, list[str]]
    package_dir: Incomplete
    py_modules: Incomplete
    libraries: Incomplete
    headers: Incomplete
    ext_modules: Incomplete
    ext_package: Incomplete
    include_dirs: Incomplete
    extra_path: Incomplete
    scripts: Incomplete
    data_files: Incomplete
    password: str
    command_obj: Incomplete
    have_run: Incomplete
    want_user_cfg: bool
    def dump_option_dicts(
        self, header: Incomplete | None = None, commands: Incomplete | None = None, indent: str = ""
    ) -> None: ...
    def find_config_files(self): ...
    commands: Incomplete
    def parse_command_line(self): ...
    def finalize_options(self) -> None: ...
    def handle_display_options(self, option_order): ...
    def print_command_list(self, commands, header, max_length) -> None: ...
    def print_commands(self) -> None: ...
    def get_command_list(self): ...
    def get_command_packages(self): ...
    def get_command_class(self, command: str) -> type[Command]: ...
    @overload
    def reinitialize_command(self, command: str, reinit_subcommands: bool = False) -> Command: ...
    @overload
    def reinitialize_command(self, command: _CommandT, reinit_subcommands: bool = False) -> _CommandT: ...
    def announce(self, msg, level: int = 2) -> None: ...
    def run_commands(self) -> None: ...
    def run_command(self, command: str) -> None: ...
    def has_pure_modules(self) -> bool: ...
    def has_ext_modules(self) -> bool: ...
    def has_c_libraries(self) -> bool: ...
    def has_modules(self) -> bool: ...
    def has_headers(self) -> bool: ...
    def has_scripts(self) -> bool: ...
    def has_data_files(self) -> bool: ...
    def is_pure(self) -> bool: ...

    # Getter methods generated in __init__
    def get_name(self) -> str: ...
    def get_version(self) -> str: ...
    def get_fullname(self) -> str: ...
    def get_author(self) -> str: ...
    def get_author_email(self) -> str: ...
    def get_maintainer(self) -> str: ...
    def get_maintainer_email(self) -> str: ...
    def get_contact(self) -> str: ...
    def get_contact_email(self) -> str: ...
    def get_url(self) -> str: ...
    def get_license(self) -> str: ...
    def get_licence(self) -> str: ...
    def get_description(self) -> str: ...
    def get_long_description(self) -> str: ...
    def get_keywords(self) -> str | list[str]: ...
    def get_platforms(self) -> str | list[str]: ...
    def get_classifiers(self) -> str | list[str]: ...
    def get_download_url(self) -> str: ...
    def get_requires(self) -> list[str]: ...
    def get_provides(self) -> list[str]: ...
    def get_obsoletes(self) -> list[str]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ActionResult:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ActionResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ActionResult.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Success: _ActionResult.ValueType  # 1
    NotSupported: _ActionResult.ValueType  # 2
    Error: _ActionResult.ValueType  # 3
    CantQueueThatOrder: _ActionResult.ValueType  # 4
    Retry: _ActionResult.ValueType  # 5
    Cooldown: _ActionResult.ValueType  # 6
    QueueIsFull: _ActionResult.ValueType  # 7
    RallyQueueIsFull: _ActionResult.ValueType  # 8
    NotEnoughMinerals: _ActionResult.ValueType  # 9
    NotEnoughVespene: _ActionResult.ValueType  # 10
    NotEnoughTerrazine: _ActionResult.ValueType  # 11
    NotEnoughCustom: _ActionResult.ValueType  # 12
    NotEnoughFood: _ActionResult.ValueType  # 13
    FoodUsageImpossible: _ActionResult.ValueType  # 14
    NotEnoughLife: _ActionResult.ValueType  # 15
    NotEnoughShields: _ActionResult.ValueType  # 16
    NotEnoughEnergy: _ActionResult.ValueType  # 17
    LifeSuppressed: _ActionResult.ValueType  # 18
    ShieldsSuppressed: _ActionResult.ValueType  # 19
    EnergySuppressed: _ActionResult.ValueType  # 20
    NotEnoughCharges: _ActionResult.ValueType  # 21
    CantAddMoreCharges: _ActionResult.ValueType  # 22
    TooMuchMinerals: _ActionResult.ValueType  # 23
    TooMuchVespene: _ActionResult.ValueType  # 24
    TooMuchTerrazine: _ActionResult.ValueType  # 25
    TooMuchCustom: _ActionResult.ValueType  # 26
    TooMuchFood: _ActionResult.ValueType  # 27
    TooMuchLife: _ActionResult.ValueType  # 28
    TooMuchShields: _ActionResult.ValueType  # 29
    TooMuchEnergy: _ActionResult.ValueType  # 30
    MustTargetUnitWithLife: _ActionResult.ValueType  # 31
    MustTargetUnitWithShields: _ActionResult.ValueType  # 32
    MustTargetUnitWithEnergy: _ActionResult.ValueType  # 33
    CantTrade: _ActionResult.ValueType  # 34
    CantSpend: _ActionResult.ValueType  # 35
    CantTargetThatUnit: _ActionResult.ValueType  # 36
    CouldntAllocateUnit: _ActionResult.ValueType  # 37
    UnitCantMove: _ActionResult.ValueType  # 38
    TransportIsHoldingPosition: _ActionResult.ValueType  # 39
    BuildTechRequirementsNotMet: _ActionResult.ValueType  # 40
    CantFindPlacementLocation: _ActionResult.ValueType  # 41
    CantBuildOnThat: _ActionResult.ValueType  # 42
    CantBuildTooCloseToDropOff: _ActionResult.ValueType  # 43
    CantBuildLocationInvalid: _ActionResult.ValueType  # 44
    CantSeeBuildLocation: _ActionResult.ValueType  # 45
    CantBuildTooCloseToCreepSource: _ActionResult.ValueType  # 46
    CantBuildTooCloseToResources: _ActionResult.ValueType  # 47
    CantBuildTooFarFromWater: _ActionResult.ValueType  # 48
    CantBuildTooFarFromCreepSource: _ActionResult.ValueType  # 49
    CantBuildTooFarFromBuildPowerSource: _ActionResult.ValueType  # 50
    CantBuildOnDenseTerrain: _ActionResult.ValueType  # 51
    CantTrainTooFarFromTrainPowerSource: _ActionResult.ValueType  # 52
    CantLandLocationInvalid: _ActionResult.ValueType  # 53
    CantSeeLandLocation: _ActionResult.ValueType  # 54
    CantLandTooCloseToCreepSource: _ActionResult.ValueType  # 55
    CantLandTooCloseToResources: _ActionResult.ValueType  # 56
    CantLandTooFarFromWater: _ActionResult.ValueType  # 57
    CantLandTooFarFromCreepSource: _ActionResult.ValueType  # 58
    CantLandTooFarFromBuildPowerSource: _ActionResult.ValueType  # 59
    CantLandTooFarFromTrainPowerSource: _ActionResult.ValueType  # 60
    CantLandOnDenseTerrain: _ActionResult.ValueType  # 61
    AddOnTooFarFromBuilding: _ActionResult.ValueType  # 62
    MustBuildRefineryFirst: _ActionResult.ValueType  # 63
    BuildingIsUnderConstruction: _ActionResult.ValueType  # 64
    CantFindDropOff: _ActionResult.ValueType  # 65
    CantLoadOtherPlayersUnits: _ActionResult.ValueType  # 66
    NotEnoughRoomToLoadUnit: _ActionResult.ValueType  # 67
    CantUnloadUnitsThere: _ActionResult.ValueType  # 68
    CantWarpInUnitsThere: _ActionResult.ValueType  # 69
    CantLoadImmobileUnits: _ActionResult.ValueType  # 70
    CantRechargeImmobileUnits: _ActionResult.ValueType  # 71
    CantRechargeUnderConstructionUnits: _ActionResult.ValueType  # 72
    CantLoadThatUnit: _ActionResult.ValueType  # 73
    NoCargoToUnload: _ActionResult.ValueType  # 74
    LoadAllNoTargetsFound: _ActionResult.ValueType  # 75
    NotWhileOccupied: _ActionResult.ValueType  # 76
    CantAttackWithoutAmmo: _ActionResult.ValueType  # 77
    CantHoldAnyMoreAmmo: _ActionResult.ValueType  # 78
    TechRequirementsNotMet: _ActionResult.ValueType  # 79
    MustLockdownUnitFirst: _ActionResult.ValueType  # 80
    MustTargetUnit: _ActionResult.ValueType  # 81
    MustTargetInventory: _ActionResult.ValueType  # 82
    MustTargetVisibleUnit: _ActionResult.ValueType  # 83
    MustTargetVisibleLocation: _ActionResult.ValueType  # 84
    MustTargetWalkableLocation: _ActionResult.ValueType  # 85
    MustTargetPawnableUnit: _ActionResult.ValueType  # 86
    YouCantControlThatUnit: _ActionResult.ValueType  # 87
    YouCantIssueCommandsToThatUnit: _ActionResult.ValueType  # 88
    MustTargetResources: _ActionResult.ValueType  # 89
    RequiresHealTarget: _ActionResult.ValueType  # 90
    RequiresRepairTarget: _ActionResult.ValueType  # 91
    NoItemsToDrop: _ActionResult.ValueType  # 92
    CantHoldAnyMoreItems: _ActionResult.ValueType  # 93
    CantHoldThat: _ActionResult.ValueType  # 94
    TargetHasNoInventory: _ActionResult.ValueType  # 95
    CantDropThisItem: _ActionResult.ValueType  # 96
    CantMoveThisItem: _ActionResult.ValueType  # 97
    CantPawnThisUnit: _ActionResult.ValueType  # 98
    MustTargetCaster: _ActionResult.ValueType  # 99
    CantTargetCaster: _ActionResult.ValueType  # 100
    MustTargetOuter: _ActionResult.ValueType  # 101
    CantTargetOuter: _ActionResult.ValueType  # 102
    MustTargetYourOwnUnits: _ActionResult.ValueType  # 103
    CantTargetYourOwnUnits: _ActionResult.ValueType  # 104
    MustTargetFriendlyUnits: _ActionResult.ValueType  # 105
    CantTargetFriendlyUnits: _ActionResult.ValueType  # 106
    MustTargetNeutralUnits: _ActionResult.ValueType  # 107
    CantTargetNeutralUnits: _ActionResult.ValueType  # 108
    MustTargetEnemyUnits: _ActionResult.ValueType  # 109
    CantTargetEnemyUnits: _ActionResult.ValueType  # 110
    MustTargetAirUnits: _ActionResult.ValueType  # 111
    CantTargetAirUnits: _ActionResult.ValueType  # 112
    MustTargetGroundUnits: _ActionResult.ValueType  # 113
    CantTargetGroundUnits: _ActionResult.ValueType  # 114
    MustTargetStructures: _ActionResult.ValueType  # 115
    CantTargetStructures: _ActionResult.ValueType  # 116
    MustTargetLightUnits: _ActionResult.ValueType  # 117
    CantTargetLightUnits: _ActionResult.ValueType  # 118
    MustTargetArmoredUnits: _ActionResult.ValueType  # 119
    CantTargetArmoredUnits: _ActionResult.ValueType  # 120
    MustTargetBiologicalUnits: _ActionResult.ValueType  # 121
    CantTargetBiologicalUnits: _ActionResult.ValueType  # 122
    MustTargetHeroicUnits: _ActionResult.ValueType  # 123
    CantTargetHeroicUnits: _ActionResult.ValueType  # 124
    MustTargetRoboticUnits: _ActionResult.ValueType  # 125
    CantTargetRoboticUnits: _ActionResult.ValueType  # 126
    MustTargetMechanicalUnits: _ActionResult.ValueType  # 127
    CantTargetMechanicalUnits: _ActionResult.ValueType  # 128
    MustTargetPsionicUnits: _ActionResult.ValueType  # 129
    CantTargetPsionicUnits: _ActionResult.ValueType  # 130
    MustTargetMassiveUnits: _ActionResult.ValueType  # 131
    CantTargetMassiveUnits: _ActionResult.ValueType  # 132
    MustTargetMissile: _ActionResult.ValueType  # 133
    CantTargetMissile: _ActionResult.ValueType  # 134
    MustTargetWorkerUnits: _ActionResult.ValueType  # 135
    CantTargetWorkerUnits: _ActionResult.ValueType  # 136
    MustTargetEnergyCapableUnits: _ActionResult.ValueType  # 137
    CantTargetEnergyCapableUnits: _ActionResult.ValueType  # 138
    MustTargetShieldCapableUnits: _ActionResult.ValueType  # 139
    CantTargetShieldCapableUnits: _ActionResult.ValueType  # 140
    MustTargetFlyers: _ActionResult.ValueType  # 141
    CantTargetFlyers: _ActionResult.ValueType  # 142
    MustTargetBuriedUnits: _ActionResult.ValueType  # 143
    CantTargetBuriedUnits: _ActionResult.ValueType  # 144
    MustTargetCloakedUnits: _ActionResult.ValueType  # 145
    CantTargetCloakedUnits: _ActionResult.ValueType  # 146
    MustTargetUnitsInAStasisField: _ActionResult.ValueType  # 147
    CantTargetUnitsInAStasisField: _ActionResult.ValueType  # 148
    MustTargetUnderConstructionUnits: _ActionResult.ValueType  # 149
    CantTargetUnderConstructionUnits: _ActionResult.ValueType  # 150
    MustTargetDeadUnits: _ActionResult.ValueType  # 151
    CantTargetDeadUnits: _ActionResult.ValueType  # 152
    MustTargetRevivableUnits: _ActionResult.ValueType  # 153
    CantTargetRevivableUnits: _ActionResult.ValueType  # 154
    MustTargetHiddenUnits: _ActionResult.ValueType  # 155
    CantTargetHiddenUnits: _ActionResult.ValueType  # 156
    CantRechargeOtherPlayersUnits: _ActionResult.ValueType  # 157
    MustTargetHallucinations: _ActionResult.ValueType  # 158
    CantTargetHallucinations: _ActionResult.ValueType  # 159
    MustTargetInvulnerableUnits: _ActionResult.ValueType  # 160
    CantTargetInvulnerableUnits: _ActionResult.ValueType  # 161
    MustTargetDetectedUnits: _ActionResult.ValueType  # 162
    CantTargetDetectedUnits: _ActionResult.ValueType  # 163
    CantTargetUnitWithEnergy: _ActionResult.ValueType  # 164
    CantTargetUnitWithShields: _ActionResult.ValueType  # 165
    MustTargetUncommandableUnits: _ActionResult.ValueType  # 166
    CantTargetUncommandableUnits: _ActionResult.ValueType  # 167
    MustTargetPreventDefeatUnits: _ActionResult.ValueType  # 168
    CantTargetPreventDefeatUnits: _ActionResult.ValueType  # 169
    MustTargetPreventRevealUnits: _ActionResult.ValueType  # 170
    CantTargetPreventRevealUnits: _ActionResult.ValueType  # 171
    MustTargetPassiveUnits: _ActionResult.ValueType  # 172
    CantTargetPassiveUnits: _ActionResult.ValueType  # 173
    MustTargetStunnedUnits: _ActionResult.ValueType  # 174
    CantTargetStunnedUnits: _ActionResult.ValueType  # 175
    MustTargetSummonedUnits: _ActionResult.ValueType  # 176
    CantTargetSummonedUnits: _ActionResult.ValueType  # 177
    MustTargetUser1: _ActionResult.ValueType  # 178
    CantTargetUser1: _ActionResult.ValueType  # 179
    MustTargetUnstoppableUnits: _ActionResult.ValueType  # 180
    CantTargetUnstoppableUnits: _ActionResult.ValueType  # 181
    MustTargetResistantUnits: _ActionResult.ValueType  # 182
    CantTargetResistantUnits: _ActionResult.ValueType  # 183
    MustTargetDazedUnits: _ActionResult.ValueType  # 184
    CantTargetDazedUnits: _ActionResult.ValueType  # 185
    CantLockdown: _ActionResult.ValueType  # 186
    CantMindControl: _ActionResult.ValueType  # 187
    MustTargetDestructibles: _ActionResult.ValueType  # 188
    CantTargetDestructibles: _ActionResult.ValueType  # 189
    MustTargetItems: _ActionResult.ValueType  # 190
    CantTargetItems: _ActionResult.ValueType  # 191
    NoCalldownAvailable: _ActionResult.ValueType  # 192
    WaypointListFull: _ActionResult.ValueType  # 193
    MustTargetRace: _ActionResult.ValueType  # 194
    CantTargetRace: _ActionResult.ValueType  # 195
    MustTargetSimilarUnits: _ActionResult.ValueType  # 196
    CantTargetSimilarUnits: _ActionResult.ValueType  # 197
    CantFindEnoughTargets: _ActionResult.ValueType  # 198
    AlreadySpawningLarva: _ActionResult.ValueType  # 199
    CantTargetExhaustedResources: _ActionResult.ValueType  # 200
    CantUseMinimap: _ActionResult.ValueType  # 201
    CantUseInfoPanel: _ActionResult.ValueType  # 202
    OrderQueueIsFull: _ActionResult.ValueType  # 203
    CantHarvestThatResource: _ActionResult.ValueType  # 204
    HarvestersNotRequired: _ActionResult.ValueType  # 205
    AlreadyTargeted: _ActionResult.ValueType  # 206
    CantAttackWeaponsDisabled: _ActionResult.ValueType  # 207
    CouldntReachTarget: _ActionResult.ValueType  # 208
    TargetIsOutOfRange: _ActionResult.ValueType  # 209
    TargetIsTooClose: _ActionResult.ValueType  # 210
    TargetIsOutOfArc: _ActionResult.ValueType  # 211
    CantFindTeleportLocation: _ActionResult.ValueType  # 212
    InvalidItemClass: _ActionResult.ValueType  # 213
    CantFindCancelOrder: _ActionResult.ValueType  # 214

class ActionResult(_ActionResult, metaclass=_ActionResultEnumTypeWrapper): ...

Success: ActionResult.ValueType  # 1
NotSupported: ActionResult.ValueType  # 2
Error: ActionResult.ValueType  # 3
CantQueueThatOrder: ActionResult.ValueType  # 4
Retry: ActionResult.ValueType  # 5
Cooldown: ActionResult.ValueType  # 6
QueueIsFull: ActionResult.ValueType  # 7
RallyQueueIsFull: ActionResult.ValueType  # 8
NotEnoughMinerals: ActionResult.ValueType  # 9
NotEnoughVespene: ActionResult.ValueType  # 10
NotEnoughTerrazine: ActionResult.ValueType  # 11
NotEnoughCustom: ActionResult.ValueType  # 12
NotEnoughFood: ActionResult.ValueType  # 13
FoodUsageImpossible: ActionResult.ValueType  # 14
NotEnoughLife: ActionResult.ValueType  # 15
NotEnoughShields: ActionResult.ValueType  # 16
NotEnoughEnergy: ActionResult.ValueType  # 17
LifeSuppressed: ActionResult.ValueType  # 18
ShieldsSuppressed: ActionResult.ValueType  # 19
EnergySuppressed: ActionResult.ValueType  # 20
NotEnoughCharges: ActionResult.ValueType  # 21
CantAddMoreCharges: ActionResult.ValueType  # 22
TooMuchMinerals: ActionResult.ValueType  # 23
TooMuchVespene: ActionResult.ValueType  # 24
TooMuchTerrazine: ActionResult.ValueType  # 25
TooMuchCustom: ActionResult.ValueType  # 26
TooMuchFood: ActionResult.ValueType  # 27
TooMuchLife: ActionResult.ValueType  # 28
TooMuchShields: ActionResult.ValueType  # 29
TooMuchEnergy: ActionResult.ValueType  # 30
MustTargetUnitWithLife: ActionResult.ValueType  # 31
MustTargetUnitWithShields: ActionResult.ValueType  # 32
MustTargetUnitWithEnergy: ActionResult.ValueType  # 33
CantTrade: ActionResult.ValueType  # 34
CantSpend: ActionResult.ValueType  # 35
CantTargetThatUnit: ActionResult.ValueType  # 36
CouldntAllocateUnit: ActionResult.ValueType  # 37
UnitCantMove: ActionResult.ValueType  # 38
TransportIsHoldingPosition: ActionResult.ValueType  # 39
BuildTechRequirementsNotMet: ActionResult.ValueType  # 40
CantFindPlacementLocation: ActionResult.ValueType  # 41
CantBuildOnThat: ActionResult.ValueType  # 42
CantBuildTooCloseToDropOff: ActionResult.ValueType  # 43
CantBuildLocationInvalid: ActionResult.ValueType  # 44
CantSeeBuildLocation: ActionResult.ValueType  # 45
CantBuildTooCloseToCreepSource: ActionResult.ValueType  # 46
CantBuildTooCloseToResources: ActionResult.ValueType  # 47
CantBuildTooFarFromWater: ActionResult.ValueType  # 48
CantBuildTooFarFromCreepSource: ActionResult.ValueType  # 49
CantBuildTooFarFromBuildPowerSource: ActionResult.ValueType  # 50
CantBuildOnDenseTerrain: ActionResult.ValueType  # 51
CantTrainTooFarFromTrainPowerSource: ActionResult.ValueType  # 52
CantLandLocationInvalid: ActionResult.ValueType  # 53
CantSeeLandLocation: ActionResult.ValueType  # 54
CantLandTooCloseToCreepSource: ActionResult.ValueType  # 55
CantLandTooCloseToResources: ActionResult.ValueType  # 56
CantLandTooFarFromWater: ActionResult.ValueType  # 57
CantLandTooFarFromCreepSource: ActionResult.ValueType  # 58
CantLandTooFarFromBuildPowerSource: ActionResult.ValueType  # 59
CantLandTooFarFromTrainPowerSource: ActionResult.ValueType  # 60
CantLandOnDenseTerrain: ActionResult.ValueType  # 61
AddOnTooFarFromBuilding: ActionResult.ValueType  # 62
MustBuildRefineryFirst: ActionResult.ValueType  # 63
BuildingIsUnderConstruction: ActionResult.ValueType  # 64
CantFindDropOff: ActionResult.ValueType  # 65
CantLoadOtherPlayersUnits: ActionResult.ValueType  # 66
NotEnoughRoomToLoadUnit: ActionResult.ValueType  # 67
CantUnloadUnitsThere: ActionResult.ValueType  # 68
CantWarpInUnitsThere: ActionResult.ValueType  # 69
CantLoadImmobileUnits: ActionResult.ValueType  # 70
CantRechargeImmobileUnits: ActionResult.ValueType  # 71
CantRechargeUnderConstructionUnits: ActionResult.ValueType  # 72
CantLoadThatUnit: ActionResult.ValueType  # 73
NoCargoToUnload: ActionResult.ValueType  # 74
LoadAllNoTargetsFound: ActionResult.ValueType  # 75
NotWhileOccupied: ActionResult.ValueType  # 76
CantAttackWithoutAmmo: ActionResult.ValueType  # 77
CantHoldAnyMoreAmmo: ActionResult.ValueType  # 78
TechRequirementsNotMet: ActionResult.ValueType  # 79
MustLockdownUnitFirst: ActionResult.ValueType  # 80
MustTargetUnit: ActionResult.ValueType  # 81
MustTargetInventory: ActionResult.ValueType  # 82
MustTargetVisibleUnit: ActionResult.ValueType  # 83
MustTargetVisibleLocation: ActionResult.ValueType  # 84
MustTargetWalkableLocation: ActionResult.ValueType  # 85
MustTargetPawnableUnit: ActionResult.ValueType  # 86
YouCantControlThatUnit: ActionResult.ValueType  # 87
YouCantIssueCommandsToThatUnit: ActionResult.ValueType  # 88
MustTargetResources: ActionResult.ValueType  # 89
RequiresHealTarget: ActionResult.ValueType  # 90
RequiresRepairTarget: ActionResult.ValueType  # 91
NoItemsToDrop: ActionResult.ValueType  # 92
CantHoldAnyMoreItems: ActionResult.ValueType  # 93
CantHoldThat: ActionResult.ValueType  # 94
TargetHasNoInventory: ActionResult.ValueType  # 95
CantDropThisItem: ActionResult.ValueType  # 96
CantMoveThisItem: ActionResult.ValueType  # 97
CantPawnThisUnit: ActionResult.ValueType  # 98
MustTargetCaster: ActionResult.ValueType  # 99
CantTargetCaster: ActionResult.ValueType  # 100
MustTargetOuter: ActionResult.ValueType  # 101
CantTargetOuter: ActionResult.ValueType  # 102
MustTargetYourOwnUnits: ActionResult.ValueType  # 103
CantTargetYourOwnUnits: ActionResult.ValueType  # 104
MustTargetFriendlyUnits: ActionResult.ValueType  # 105
CantTargetFriendlyUnits: ActionResult.ValueType  # 106
MustTargetNeutralUnits: ActionResult.ValueType  # 107
CantTargetNeutralUnits: ActionResult.ValueType  # 108
MustTargetEnemyUnits: ActionResult.ValueType  # 109
CantTargetEnemyUnits: ActionResult.ValueType  # 110
MustTargetAirUnits: ActionResult.ValueType  # 111
CantTargetAirUnits: ActionResult.ValueType  # 112
MustTargetGroundUnits: ActionResult.ValueType  # 113
CantTargetGroundUnits: ActionResult.ValueType  # 114
MustTargetStructures: ActionResult.ValueType  # 115
CantTargetStructures: ActionResult.ValueType  # 116
MustTargetLightUnits: ActionResult.ValueType  # 117
CantTargetLightUnits: ActionResult.ValueType  # 118
MustTargetArmoredUnits: ActionResult.ValueType  # 119
CantTargetArmoredUnits: ActionResult.ValueType  # 120
MustTargetBiologicalUnits: ActionResult.ValueType  # 121
CantTargetBiologicalUnits: ActionResult.ValueType  # 122
MustTargetHeroicUnits: ActionResult.ValueType  # 123
CantTargetHeroicUnits: ActionResult.ValueType  # 124
MustTargetRoboticUnits: ActionResult.ValueType  # 125
CantTargetRoboticUnits: ActionResult.ValueType  # 126
MustTargetMechanicalUnits: ActionResult.ValueType  # 127
CantTargetMechanicalUnits: ActionResult.ValueType  # 128
MustTargetPsionicUnits: ActionResult.ValueType  # 129
CantTargetPsionicUnits: ActionResult.ValueType  # 130
MustTargetMassiveUnits: ActionResult.ValueType  # 131
CantTargetMassiveUnits: ActionResult.ValueType  # 132
MustTargetMissile: ActionResult.ValueType  # 133
CantTargetMissile: ActionResult.ValueType  # 134
MustTargetWorkerUnits: ActionResult.ValueType  # 135
CantTargetWorkerUnits: ActionResult.ValueType  # 136
MustTargetEnergyCapableUnits: ActionResult.ValueType  # 137
CantTargetEnergyCapableUnits: ActionResult.ValueType  # 138
MustTargetShieldCapableUnits: ActionResult.ValueType  # 139
CantTargetShieldCapableUnits: ActionResult.ValueType  # 140
MustTargetFlyers: ActionResult.ValueType  # 141
CantTargetFlyers: ActionResult.ValueType  # 142
MustTargetBuriedUnits: ActionResult.ValueType  # 143
CantTargetBuriedUnits: ActionResult.ValueType  # 144
MustTargetCloakedUnits: ActionResult.ValueType  # 145
CantTargetCloakedUnits: ActionResult.ValueType  # 146
MustTargetUnitsInAStasisField: ActionResult.ValueType  # 147
CantTargetUnitsInAStasisField: ActionResult.ValueType  # 148
MustTargetUnderConstructionUnits: ActionResult.ValueType  # 149
CantTargetUnderConstructionUnits: ActionResult.ValueType  # 150
MustTargetDeadUnits: ActionResult.ValueType  # 151
CantTargetDeadUnits: ActionResult.ValueType  # 152
MustTargetRevivableUnits: ActionResult.ValueType  # 153
CantTargetRevivableUnits: ActionResult.ValueType  # 154
MustTargetHiddenUnits: ActionResult.ValueType  # 155
CantTargetHiddenUnits: ActionResult.ValueType  # 156
CantRechargeOtherPlayersUnits: ActionResult.ValueType  # 157
MustTargetHallucinations: ActionResult.ValueType  # 158
CantTargetHallucinations: ActionResult.ValueType  # 159
MustTargetInvulnerableUnits: ActionResult.ValueType  # 160
CantTargetInvulnerableUnits: ActionResult.ValueType  # 161
MustTargetDetectedUnits: ActionResult.ValueType  # 162
CantTargetDetectedUnits: ActionResult.ValueType  # 163
CantTargetUnitWithEnergy: ActionResult.ValueType  # 164
CantTargetUnitWithShields: ActionResult.ValueType  # 165
MustTargetUncommandableUnits: ActionResult.ValueType  # 166
CantTargetUncommandableUnits: ActionResult.ValueType  # 167
MustTargetPreventDefeatUnits: ActionResult.ValueType  # 168
CantTargetPreventDefeatUnits: ActionResult.ValueType  # 169
MustTargetPreventRevealUnits: ActionResult.ValueType  # 170
CantTargetPreventRevealUnits: ActionResult.ValueType  # 171
MustTargetPassiveUnits: ActionResult.ValueType  # 172
CantTargetPassiveUnits: ActionResult.ValueType  # 173
MustTargetStunnedUnits: ActionResult.ValueType  # 174
CantTargetStunnedUnits: ActionResult.ValueType  # 175
MustTargetSummonedUnits: ActionResult.ValueType  # 176
CantTargetSummonedUnits: ActionResult.ValueType  # 177
MustTargetUser1: ActionResult.ValueType  # 178
CantTargetUser1: ActionResult.ValueType  # 179
MustTargetUnstoppableUnits: ActionResult.ValueType  # 180
CantTargetUnstoppableUnits: ActionResult.ValueType  # 181
MustTargetResistantUnits: ActionResult.ValueType  # 182
CantTargetResistantUnits: ActionResult.ValueType  # 183
MustTargetDazedUnits: ActionResult.ValueType  # 184
CantTargetDazedUnits: ActionResult.ValueType  # 185
CantLockdown: ActionResult.ValueType  # 186
CantMindControl: ActionResult.ValueType  # 187
MustTargetDestructibles: ActionResult.ValueType  # 188
CantTargetDestructibles: ActionResult.ValueType  # 189
MustTargetItems: ActionResult.ValueType  # 190
CantTargetItems: ActionResult.ValueType  # 191
NoCalldownAvailable: ActionResult.ValueType  # 192
WaypointListFull: ActionResult.ValueType  # 193
MustTargetRace: ActionResult.ValueType  # 194
CantTargetRace: ActionResult.ValueType  # 195
MustTargetSimilarUnits: ActionResult.ValueType  # 196
CantTargetSimilarUnits: ActionResult.ValueType  # 197
CantFindEnoughTargets: ActionResult.ValueType  # 198
AlreadySpawningLarva: ActionResult.ValueType  # 199
CantTargetExhaustedResources: ActionResult.ValueType  # 200
CantUseMinimap: ActionResult.ValueType  # 201
CantUseInfoPanel: ActionResult.ValueType  # 202
OrderQueueIsFull: ActionResult.ValueType  # 203
CantHarvestThatResource: ActionResult.ValueType  # 204
HarvestersNotRequired: ActionResult.ValueType  # 205
AlreadyTargeted: ActionResult.ValueType  # 206
CantAttackWeaponsDisabled: ActionResult.ValueType  # 207
CouldntReachTarget: ActionResult.ValueType  # 208
TargetIsOutOfRange: ActionResult.ValueType  # 209
TargetIsTooClose: ActionResult.ValueType  # 210
TargetIsOutOfArc: ActionResult.ValueType  # 211
CantFindTeleportLocation: ActionResult.ValueType  # 212
InvalidItemClass: ActionResult.ValueType  # 213
CantFindCancelOrder: ActionResult.ValueType  # 214
global___ActionResult = ActionResult



import logging
from _typeshed import ReadableBuffer
from collections.abc import Callable, Generator
from typing import Any

from serial.serialutil import SerialBase

LOGGER_LEVELS: dict[str, int]
SE: bytes
NOP: bytes
DM: bytes
BRK: bytes
IP: bytes
AO: bytes
AYT: bytes
EC: bytes
EL: bytes
GA: bytes
SB: bytes
WILL: bytes
WONT: bytes
DO: bytes
DONT: bytes
IAC: bytes
IAC_DOUBLED: bytes
BINARY: bytes
ECHO: bytes
SGA: bytes
COM_PORT_OPTION: bytes
SET_BAUDRATE: bytes
SET_DATASIZE: bytes
SET_PARITY: bytes
SET_STOPSIZE: bytes
SET_CONTROL: bytes
NOTIFY_LINESTATE: bytes
NOTIFY_MODEMSTATE: bytes
FLOWCONTROL_SUSPEND: bytes
FLOWCONTROL_RESUME: bytes
SET_LINESTATE_MASK: bytes
SET_MODEMSTATE_MASK: bytes
PURGE_DATA: bytes
SERVER_SET_BAUDRATE: bytes
SERVER_SET_DATASIZE: bytes
SERVER_SET_PARITY: bytes
SERVER_SET_STOPSIZE: bytes
SERVER_SET_CONTROL: bytes
SERVER_NOTIFY_LINESTATE: bytes
SERVER_NOTIFY_MODEMSTATE: bytes
SERVER_FLOWCONTROL_SUSPEND: bytes
SERVER_FLOWCONTROL_RESUME: bytes
SERVER_SET_LINESTATE_MASK: bytes
SERVER_SET_MODEMSTATE_MASK: bytes
SERVER_PURGE_DATA: bytes
RFC2217_ANSWER_MAP: dict[bytes, bytes]
SET_CONTROL_REQ_FLOW_SETTING: bytes
SET_CONTROL_USE_NO_FLOW_CONTROL: bytes
SET_CONTROL_USE_SW_FLOW_CONTROL: bytes
SET_CONTROL_USE_HW_FLOW_CONTROL: bytes
SET_CONTROL_REQ_BREAK_STATE: bytes
SET_CONTROL_BREAK_ON: bytes
SET_CONTROL_BREAK_OFF: bytes
SET_CONTROL_REQ_DTR: bytes
SET_CONTROL_DTR_ON: bytes
SET_CONTROL_DTR_OFF: bytes
SET_CONTROL_REQ_RTS: bytes
SET_CONTROL_RTS_ON: bytes
SET_CONTROL_RTS_OFF: bytes
SET_CONTROL_REQ_FLOW_SETTING_IN: bytes
SET_CONTROL_USE_NO_FLOW_CONTROL_IN: bytes
SET_CONTROL_USE_SW_FLOW_CONTOL_IN: bytes
SET_CONTROL_USE_HW_FLOW_CONTOL_IN: bytes
SET_CONTROL_USE_DCD_FLOW_CONTROL: bytes
SET_CONTROL_USE_DTR_FLOW_CONTROL: bytes
SET_CONTROL_USE_DSR_FLOW_CONTROL: bytes
LINESTATE_MASK_TIMEOUT: int
LINESTATE_MASK_SHIFTREG_EMPTY: int
LINESTATE_MASK_TRANSREG_EMPTY: int
LINESTATE_MASK_BREAK_DETECT: int
LINESTATE_MASK_FRAMING_ERROR: int
LINESTATE_MASK_PARTIY_ERROR: int
LINESTATE_MASK_OVERRUN_ERROR: int
LINESTATE_MASK_DATA_READY: int
MODEMSTATE_MASK_CD: int
MODEMSTATE_MASK_RI: int
MODEMSTATE_MASK_DSR: int
MODEMSTATE_MASK_CTS: int
MODEMSTATE_MASK_CD_CHANGE: int
MODEMSTATE_MASK_RI_CHANGE: int
MODEMSTATE_MASK_DSR_CHANGE: int
MODEMSTATE_MASK_CTS_CHANGE: int
PURGE_RECEIVE_BUFFER: bytes
PURGE_TRANSMIT_BUFFER: bytes
PURGE_BOTH_BUFFERS: bytes
RFC2217_PARITY_MAP: dict[str, int]
RFC2217_REVERSE_PARITY_MAP: dict[int, str]
RFC2217_STOPBIT_MAP: dict[int | float, int]
RFC2217_REVERSE_STOPBIT_MAP: dict[int, int | float]
M_NORMAL: int
M_IAC_SEEN: int
M_NEGOTIATE: int
REQUESTED: str
ACTIVE: str
INACTIVE: str
REALLY_INACTIVE: str

class TelnetOption:
    connection: Serial
    name: str
    option: bytes
    send_yes: bytes
    send_no: bytes
    ack_yes: bytes
    ack_no: bytes
    state: str
    active: bool
    activation_callback: Callable[[], Any]

    def __init__(
        self,
        connection: Serial,
        name: str,
        option: bytes,
        send_yes: bytes,
        send_no: bytes,
        ack_yes: bytes,
        ack_no: bytes,
        initial_state: str,
        activation_callback: Callable[[], Any] | None = None,
    ) -> None: ...
    def process_incoming(self, command: bytes) -> None: ...

class TelnetSubnegotiation:
    connection: Serial
    name: str
    option: bytes
    value: bytes | None
    ack_option: bytes
    state: str
    def __init__(self, connection: Serial, name: str, option: bytes, ack_option: bytes | None = None) -> None: ...
    def set(self, value: bytes) -> None: ...
    def is_ready(self) -> bool: ...
    @property
    def active(self) -> bool: ...
    def wait(self, timeout: float = 3) -> None: ...
    def check_answer(self, suboption: bytes) -> None: ...

class Serial(SerialBase):
    logger: logging.Logger | None
    def open(self) -> None: ...
    def from_url(self, url: str) -> tuple[str, int]: ...
    @property
    def in_waiting(self) -> int: ...
    def read(self, size: int = 1) -> bytes: ...
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    def reset_input_buffer(self) -> None: ...
    def reset_output_buffer(self) -> None: ...
    @property
    def cts(self) -> bool: ...
    @property
    def dsr(self) -> bool: ...
    @property
    def ri(self) -> bool: ...
    @property
    def cd(self) -> bool: ...
    def telnet_send_option(self, action: bytes, option: bytes) -> None: ...
    def rfc2217_send_subnegotiation(self, option: bytes, value: bytes = b"") -> None: ...
    def rfc2217_send_purge(self, value: bytes) -> None: ...
    def rfc2217_set_control(self, value: bytes) -> None: ...
    def rfc2217_flow_server_ready(self) -> None: ...
    def get_modem_state(self) -> int: ...

class PortManager:
    serial: Serial
    connection: Serial
    logger: logging.Logger | None
    mode: int
    suboption: bytes | None
    telnet_command: bytes | None
    modemstate_mask: int
    last_modemstate: int | None
    linstate_mask: int
    def __init__(self, serial_port: Serial, connection: Serial, logger: logging.Logger | None = None) -> None: ...
    def telnet_send_option(self, action: bytes, option: bytes) -> None: ...
    def rfc2217_send_subnegotiation(self, option: bytes, value: bytes = b"") -> None: ...
    def check_modem_lines(self, force_notification: bool = False) -> None: ...
    def escape(self, data: bytes) -> Generator[bytes, None, None]: ...
    def filter(self, data: bytes) -> Generator[bytes, None, None]: ...



import decimal
from _typeshed import Incomplete
from collections import OrderedDict
from collections.abc import Callable, Generator, Iterator, Sequence
from contextlib import contextmanager
from re import Pattern
from typing import Any, ClassVar, Literal, NamedTuple, TypeVar, overload
from typing_extensions import Self, TypeAlias

from .syntax import Name, Raw

__pdoc__: dict[str, bool]

_CallableT = TypeVar("_CallableT", bound=Callable[..., Any])

def force_nodocument(item: _CallableT) -> _CallableT: ...
def force_document(item: _CallableT) -> _CallableT: ...

Number: TypeAlias = int | float | decimal.Decimal
NumberClass: tuple[type, ...]
WHITESPACE: frozenset[str]
EOL_CHARS: frozenset[str]
DELIMITERS: frozenset[str]
STR_ESC: Pattern[str]
STR_ESC_MAP: dict[str, str]

class GraphicsStateDictRegistry(OrderedDict[Raw, Name]):
    def register_style(self, style: GraphicsStyle) -> Name | None: ...

def number_to_str(number) -> str: ...
def render_pdf_primitive(primitive) -> Raw: ...

class _DeviceRGBBase(NamedTuple):
    r: Number
    g: Number
    b: Number
    a: Number | None

class DeviceRGB(_DeviceRGBBase):
    OPERATOR: ClassVar[str]
    def __new__(cls, r: Number, g: Number, b: Number, a: Number | None = None) -> Self: ...
    @property
    def colors(self) -> tuple[Number, Number, Number]: ...
    def serialize(self) -> str: ...

class _DeviceGrayBase(NamedTuple):
    g: Number
    a: Number | None

class DeviceGray(_DeviceGrayBase):
    OPERATOR: ClassVar[str]
    def __new__(cls, g: Number, a: Number | None = None) -> Self: ...
    @property
    def colors(self) -> tuple[Number]: ...
    def serialize(self) -> str: ...

class _DeviceCMYKBase(NamedTuple):
    c: Number
    m: Number
    y: Number
    k: Number
    a: Number | None

class DeviceCMYK(_DeviceCMYKBase):
    OPERATOR: ClassVar[str]
    def __new__(cls, c: Number, m: Number, y: Number, k: Number, a: Number | None = None) -> Self: ...
    @property
    def colors(self) -> tuple[Number, Number, Number, Number]: ...
    def serialize(self) -> str: ...

def rgb8(r, g, b, a: Incomplete | None = None) -> DeviceRGB: ...
def gray8(g, a: Incomplete | None = None) -> DeviceGray: ...
@overload
def convert_to_device_color(r: DeviceGray, g: int = -1, b: int = -1) -> DeviceGray: ...
@overload
def convert_to_device_color(r: DeviceRGB, g: int = -1, b: int = -1) -> DeviceRGB: ...
@overload
def convert_to_device_color(r: int, g: Literal[-1] = -1, b: Literal[-1] = -1) -> DeviceGray: ...
@overload
def convert_to_device_color(r: Sequence[int] | int, g: int, b: int) -> DeviceGray | DeviceRGB: ...
def cmyk8(c, m, y, k, a: Incomplete | None = None) -> DeviceCMYK: ...
def color_from_hex_string(hexstr) -> DeviceRGB: ...
def color_from_rgb_string(rgbstr) -> DeviceRGB: ...

class Point(NamedTuple):
    x: Number
    y: Number
    def render(self): ...
    def dot(self, other): ...
    def angle(self, other): ...
    def mag(self): ...
    def __add__(self, other): ...
    def __sub__(self, other): ...
    def __neg__(self): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __truediv__(self, other): ...
    def __floordiv__(self, other): ...
    def __matmul__(self, other): ...

class Transform(NamedTuple):
    a: Number
    b: Number
    c: Number
    d: Number
    e: Number
    f: Number
    @classmethod
    def identity(cls): ...
    @classmethod
    def translation(cls, x, y): ...
    @classmethod
    def scaling(cls, x, y: Incomplete | None = None): ...
    @classmethod
    def rotation(cls, theta): ...
    @classmethod
    def rotation_d(cls, theta_d): ...
    @classmethod
    def shearing(cls, x, y: Incomplete | None = None): ...
    def translate(self, x, y): ...
    def scale(self, x, y: Incomplete | None = None): ...
    def rotate(self, theta): ...
    def rotate_d(self, theta_d): ...
    def shear(self, x, y: Incomplete | None = None): ...
    def about(self, x, y): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __matmul__(self, other): ...
    def render(self, last_item): ...

class GraphicsStyle:
    INHERIT: ClassVar[Incomplete]
    MERGE_PROPERTIES: ClassVar[tuple[str, ...]]
    TRANSPARENCY_KEYS: ClassVar[tuple[Name, ...]]
    PDF_STYLE_KEYS: ClassVar[tuple[Name, ...]]
    @classmethod
    def merge(cls, parent, child): ...
    def __init__(self) -> None: ...
    def __deepcopy__(self, memo) -> Self: ...
    @property
    def allow_transparency(self): ...
    @allow_transparency.setter
    def allow_transparency(self, new): ...
    @property
    def paint_rule(self): ...
    @paint_rule.setter
    def paint_rule(self, new) -> None: ...
    @property
    def auto_close(self): ...
    @auto_close.setter
    def auto_close(self, new) -> None: ...
    @property
    def intersection_rule(self): ...
    @intersection_rule.setter
    def intersection_rule(self, new) -> None: ...
    @property
    def fill_color(self): ...
    @fill_color.setter
    def fill_color(self, color) -> None: ...
    @property
    def fill_opacity(self): ...
    @fill_opacity.setter
    def fill_opacity(self, new) -> None: ...
    @property
    def stroke_color(self): ...
    @stroke_color.setter
    def stroke_color(self, color) -> None: ...
    @property
    def stroke_opacity(self): ...
    @stroke_opacity.setter
    def stroke_opacity(self, new) -> None: ...
    @property
    def blend_mode(self): ...
    @blend_mode.setter
    def blend_mode(self, value) -> None: ...
    @property
    def stroke_width(self): ...
    @stroke_width.setter
    def stroke_width(self, width) -> None: ...
    @property
    def stroke_cap_style(self): ...
    @stroke_cap_style.setter
    def stroke_cap_style(self, value) -> None: ...
    @property
    def stroke_join_style(self): ...
    @stroke_join_style.setter
    def stroke_join_style(self, value) -> None: ...
    @property
    def stroke_miter_limit(self): ...
    @stroke_miter_limit.setter
    def stroke_miter_limit(self, value) -> None: ...
    @property
    def stroke_dash_pattern(self): ...
    @stroke_dash_pattern.setter
    def stroke_dash_pattern(self, value) -> None: ...
    @property
    def stroke_dash_phase(self): ...
    @stroke_dash_phase.setter
    def stroke_dash_phase(self, value): ...
    def serialize(self) -> Raw | None: ...
    def resolve_paint_rule(self): ...

class Move(NamedTuple):
    pt: Point
    @property
    def end_point(self): ...
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeMove(NamedTuple):
    pt: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class Line(NamedTuple):
    pt: Point
    @property
    def end_point(self): ...
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeLine(NamedTuple):
    pt: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class HorizontalLine(NamedTuple):
    x: Number
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeHorizontalLine(NamedTuple):
    x: Number
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class VerticalLine(NamedTuple):
    y: Number
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeVerticalLine(NamedTuple):
    y: Number
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class BezierCurve(NamedTuple):
    c1: Point
    c2: Point
    end: Point
    @property
    def end_point(self): ...
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeBezierCurve(NamedTuple):
    c1: Point
    c2: Point
    end: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class QuadraticBezierCurve(NamedTuple):
    ctrl: Point
    end: Point
    @property
    def end_point(self): ...
    def to_cubic_curve(self, start_point): ...
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeQuadraticBezierCurve(NamedTuple):
    ctrl: Point
    end: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class Arc(NamedTuple):
    radii: Point
    rotation: Number
    large: bool
    sweep: bool
    end: Point
    @staticmethod
    def subdivde_sweep(sweep_angle) -> Generator[Incomplete, None, None]: ...
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RelativeArc(NamedTuple):
    radii: Point
    rotation: Number
    large: bool
    sweep: bool
    end: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class Rectangle(NamedTuple):
    org: Point
    size: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class RoundedRectangle(NamedTuple):
    org: Point
    size: Point
    corner_radii: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class Ellipse(NamedTuple):
    radii: Point
    center: Point
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class ImplicitClose(NamedTuple):
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class Close(NamedTuple):
    def render(self, gsd_registry, style, last_item, initial_point): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class DrawingContext:
    def __init__(self) -> None: ...
    def add_item(self, item, _copy: bool = True) -> None: ...
    def render(self, gsd_registry, first_point, scale, height, starting_style): ...
    def render_debug(self, gsd_registry, first_point, scale, height, starting_style, debug_stream): ...

class PaintedPath:
    def __init__(self, x: int = 0, y: int = 0) -> None: ...
    def __deepcopy__(self, memo) -> Self: ...
    @property
    def style(self): ...
    @property
    def transform(self): ...
    @transform.setter
    def transform(self, tf) -> None: ...
    @property
    def auto_close(self): ...
    @auto_close.setter
    def auto_close(self, should) -> None: ...
    @property
    def paint_rule(self): ...
    @paint_rule.setter
    def paint_rule(self, style) -> None: ...
    @property
    def clipping_path(self): ...
    @clipping_path.setter
    def clipping_path(self, new_clipath) -> None: ...
    @contextmanager
    def transform_group(self, transform) -> Iterator[Self]: ...
    def add_path_element(self, item, _copy: bool = True) -> None: ...
    def remove_last_path_element(self) -> None: ...
    def rectangle(self, x, y, w, h, rx: int = 0, ry: int = 0) -> Self: ...
    def circle(self, cx, cy, r) -> Self: ...
    def ellipse(self, cx, cy, rx, ry) -> Self: ...
    def move_to(self, x, y) -> Self: ...
    def move_relative(self, x, y) -> Self: ...
    def line_to(self, x, y) -> Self: ...
    def line_relative(self, dx, dy) -> Self: ...
    def horizontal_line_to(self, x) -> Self: ...
    def horizontal_line_relative(self, dx) -> Self: ...
    def vertical_line_to(self, y) -> Self: ...
    def vertical_line_relative(self, dy) -> Self: ...
    def curve_to(self, x1, y1, x2, y2, x3, y3) -> Self: ...
    def curve_relative(self, dx1, dy1, dx2, dy2, dx3, dy3) -> Self: ...
    def quadratic_curve_to(self, x1, y1, x2, y2) -> Self: ...
    def quadratic_curve_relative(self, dx1, dy1, dx2, dy2) -> Self: ...
    def arc_to(self, rx, ry, rotation, large_arc, positive_sweep, x, y) -> Self: ...
    def arc_relative(self, rx, ry, rotation, large_arc, positive_sweep, dx, dy) -> Self: ...
    def close(self) -> None: ...
    def render(
        self, gsd_registry, style, last_item, initial_point, debug_stream: Incomplete | None = None, pfx: Incomplete | None = None
    ): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class ClippingPath(PaintedPath):
    paint_rule: Incomplete
    def __init__(self, x: int = 0, y: int = 0) -> None: ...
    def render(
        self, gsd_registry, style, last_item, initial_point, debug_stream: Incomplete | None = None, pfx: Incomplete | None = None
    ): ...
    def render_debug(self, gsd_registry, style, last_item, initial_point, debug_stream, pfx): ...

class GraphicsContext:
    style: GraphicsStyle
    path_items: list[Incomplete]
    def __init__(self) -> None: ...
    def __deepcopy__(self, memo) -> Self: ...
    @property
    def transform(self) -> Transform | None: ...
    @transform.setter
    def transform(self, tf) -> None: ...
    @property
    def clipping_path(self) -> ClippingPath | None: ...
    @clipping_path.setter
    def clipping_path(self, new_clipath) -> None: ...
    def add_item(self, item, _copy: bool = True) -> None: ...
    def remove_last_item(self) -> None: ...
    def merge(self, other_context) -> None: ...
    def build_render_list(
        self,
        gsd_registry,
        style,
        last_item,
        initial_point,
        debug_stream: Incomplete | None = None,
        pfx: Incomplete | None = None,
        _push_stack: bool = True,
    ): ...
    def render(
        self,
        gsd_registry,
        style: DrawingContext,
        last_item,
        initial_point,
        debug_stream: Incomplete | None = None,
        pfx: Incomplete | None = None,
        _push_stack: bool = True,
    ): ...
    def render_debug(
        self, gsd_registry, style: DrawingContext, last_item, initial_point, debug_stream, pfx, _push_stack: bool = True
    ): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def from_graph6_bytes(bytes_in): ...
def to_graph6_bytes(G, nodes: Incomplete | None = None, header: bool = True): ...
@_dispatch
def read_graph6(path): ...
def write_graph6(G, path, nodes: Incomplete | None = None, header: bool = True): ...



from typing import TypeVar

_T = TypeVar("_T")

__all__: list[str] = []

def get_unpatched(item: _T) -> _T: ...
def get_unpatched_class(cls): ...
def patch_all() -> None: ...
def patch_func(replacement, target_mod, func_name) -> None: ...
def get_unpatched_function(candidate): ...
def patch_for_msvc_specialized_compiler(): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import IslamoWesternCalendar

class Benin(IslamoWesternCalendar):
    include_labour_day: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_ascension: ClassVar[bool]
    include_whit_monday: ClassVar[bool]
    include_assumption: ClassVar[bool]
    include_all_saints: ClassVar[bool]
    include_prophet_birthday: ClassVar[bool]
    include_eid_al_fitr: ClassVar[bool]
    include_day_of_sacrifice: ClassVar[bool]
    include_day_of_sacrifice_label: ClassVar[str]
    FIXED_HOLIDAYS: Incomplete
    WEEKEND_DAYS: Incomplete



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Lease(SystemBackendMixin):
    def read_lease(self, lease_id): ...
    def list_leases(self, prefix): ...
    def renew_lease(self, lease_id, increment: Incomplete | None = None): ...
    def revoke_lease(self, lease_id): ...
    def revoke_prefix(self, prefix): ...
    def revoke_force(self, prefix): ...



from .common import (
    DefusedXmlException as DefusedXmlException,
    DTDForbidden as DTDForbidden,
    EntitiesForbidden as EntitiesForbidden,
    ExternalReferenceForbidden as ExternalReferenceForbidden,
    NotSupportedError as NotSupportedError,
)

__all__ = ["DefusedXmlException", "DTDForbidden", "EntitiesForbidden", "ExternalReferenceForbidden", "NotSupportedError"]



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def prefix_tree(paths): ...
@_dispatch
def prefix_tree_recursive(paths): ...
@_dispatch
def random_tree(n, seed: Incomplete | None = None, create_using: Incomplete | None = None): ...
